@@ -1,4 +1,3 @@
-
 /* convert.c - high level routines for LaTeX to RTF conversion
 
 Copyright (C) 2002 The Free Software Foundation
@@ -18,31 +17,6 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 This file is available from http://sourceforge.net/projects/latex2rtf/
-
-TeX has six modes:
-	
-	MODE_VERTICAL              Building the main vertical list, from which the 
-	                           pages of output are derived
-	              
-	MODE_INTERNAL_VERTICAL     Building a vertical list from a vbox
-	
-	MODE_HORIZONTAL            Building a horizontal list for a paragraph
-	
-	MODE_RESTICTED_HORIZONTAL  Building a horizontal list for an hbox
-	
-	MODE_MATH                  Building a mathematical formula to be placed in a 
-	                           horizontal list
-	                           
-	MODE_DISPLAYMATH           Building a mathematical formula to be placed on a
-	                           line by itself, temporarily interrupting the current paragraph
-	                           
-LaTeX has three modes: paragraph mode, math mode, or left-to-right mode.
-This is not a particularly useful, since paragraph mode is a combination of
-vertical and horizontal modes. 
-                         
-Why bother keeping track of modes?  Mostly so that paragraph indentation gets handled
-correctly, as well as vertical and horizontal space.
-
 */
 
 #include <string.h>
@@ -54,81 +28,44 @@ correctly, as well as vertical and horizontal space.
 #include "commands.h"
 #include "chars.h"
 #include "funct1.h"
-#include "l2r_fonts.h"
+#include "fonts.h"
 #include "stack.h"
 #include "tables.h"
-#include "equation.h"
+#include "equations.h"
 #include "direct.h"
 #include "ignore.h"
 #include "cfg.h"
-#include "encode.h"
-#include "util.h"
+#include "encodings.h"
+#include "utils.h"
 #include "parser.h"
 #include "lengths.h"
 #include "counters.h"
 #include "preamble.h"
+#include "vertical.h"
 
 static void TranslateCommand(); /* converts commands */
 
 static int ret = 0;
-static int g_TeX_mode = MODE_VERTICAL;
 
-char TexModeName[7][25] = { "bad", "internal vertical", "horizontal",
-    "restricted horizontal", "math", "displaymath", "vertical"
-};
-
-void SetTexMode(int mode)
-{
-    if (abs(mode) != g_TeX_mode)
-        diagnostics(4, "TeX mode changing from [%s] -> [%s]", TexModeName[g_TeX_mode], TexModeName[abs(mode)]);
-
-    if (mode < 0) {             /* hack to allow CmdStartParagraph to set mode directly */
-        g_TeX_mode = -mode;
-        return;
-    }
-
-    if (g_TeX_mode == MODE_VERTICAL && mode == MODE_HORIZONTAL)
-        CmdStartParagraph(ANY_PAR);
-
-    if (g_TeX_mode == MODE_HORIZONTAL && mode == MODE_VERTICAL)
-        CmdEndParagraph(0);
-
-    g_TeX_mode = mode;
-}
-
-int GetTexMode(void)
-{
-    return g_TeX_mode;
-}
-
-void ConvertString(char *string)
+void ConvertString(const char *string)
 
 /******************************************************************************
      purpose : converts string in TeX-format to Rtf-format
  ******************************************************************************/
 {
-    char temp[256];
-
     if (string == NULL)
         return;
 
-    if (strlen(string) < 250)
-        strncpy(temp, string, 250);
-    else {
-        strncpy(temp, string, 125);
-        strncpy(temp + 125, "\n...\n", 5);
-        strncpy(temp + 130, string + strlen(string) - 125, 125);
-        temp[255] = '\0';
-    }
-
     if (PushSource(NULL, string) == 0) {
-        diagnostics(4, "Entering Convert() from StringConvert()\n******\n%s\n*****", temp);
+        diagnostics(5, "Entering Convert() from ConvertString()");
+
+		show_string(5, string, "convert");
 
         while (StillSource())
             Convert();
 
         PopSource();
-        diagnostics(4, "Exiting Convert() from StringConvert()");
+        diagnostics(5, "Exiting Convert() from ConvertString()");
     }
 }
 
@@ -171,7 +108,7 @@ void ConvertAllttString(char *s)
                     break;
 
                 default:
-                    putRtfChar(cThis);
+                    fprintRTF("%c",cThis);
                     break;
             }
         }
@@ -184,31 +121,65 @@ void Convert()
 
 /****************************************************************************
 purpose: converts inputfile and writes result to outputfile
-globals: fTex, fRtf and all global flags for convert (see above)
  ****************************************************************************/
 {
     char cThis = '\n';
     char cLast = '\0';
     char cNext;
-    int mode, count, pending_new_paragraph;
+    int count, pending_new_paragraph;
 
-    diagnostics(4, "Entering Convert ret = %d", ret);
+    diagnostics(5, "Entering Convert ret = %d", ret);
     RecursionLevel++;
     PushLevels();
 
     while ((cThis = getTexChar()) && cThis != '\0') {
 
         if (cThis == '\n')
-            diagnostics(5, "Current character is '\\n' mode = %d ret = %d level = %d", GetTexMode(), ret,
+            diagnostics(6, "Current character is '\\n' mode = %d ret = %d level = %d", getTexMode(), ret,
               RecursionLevel);
         else
-            diagnostics(5, "Current character is '%c' mode = %d ret = %d level = %d", cThis, GetTexMode(), ret,
+            diagnostics(6, "Current character is '%c' mode = %d ret = %d level = %d", cThis, getTexMode(), ret,
               RecursionLevel);
 
-        mode = GetTexMode();
-
         pending_new_paragraph--;
 
+		/* preliminary support for utf8 sequences.  Thanks to CSH */
+        if ((cThis & 0x8000) && (strcmp(g_charset_encoding_name, "utf8") == 0)) {
+        	unsigned char byte;
+        	unsigned int len, value, i;
+
+        	/* Get the number of bytes in the sequence        */
+        	/* Must use an unsigned character for comparisons */
+        	byte = cThis;
+        	if (byte >= 0xF0) { 
+        		len = 3; 
+        		value = byte & ~0xF0; 
+        	} 
+        	else if (byte >= 0xE0) { 
+        		len = 2; 
+        		value = byte & ~0xE0; 
+        	}		
+        	else if (byte >= 0xC0) { 
+        		len = 1; 
+        		value = byte & ~0xC0; 
+        	}
+        	
+        	/* reassemble the character */
+        	for ( i=0; i<len; i++) {
+        		byte = getTexChar() & ~0xC0;
+        		value = (value << 6) + byte;
+        	}
+        	
+        	diagnostics(4,"(flag = 0x%X) char value = 0X%04X or %ud (%d bytes)", (unsigned char) cThis, value, value, len);
+
+        	/* values above 0x8000 must be negative! */
+			if (value < 0x8000)
+				fprintRTF("\\u%d?", value);
+			else
+				fprintRTF("\\u%d?", (int)((double)value-0x10000));
+        }        	        	
+		else
+
         switch (cThis) {
 
             case '\\':
@@ -228,8 +199,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
 
 
             case '{':
-                if (mode == MODE_VERTICAL)
-                    SetTexMode(MODE_HORIZONTAL);
+                if (getTexMode() == MODE_VERTICAL)
+                    changeTexMode(MODE_HORIZONTAL);
 
                 CleanStack();
                 PushBrace();
@@ -249,12 +220,12 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case ' ':
-                if (mode == MODE_VERTICAL || mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_VERTICAL || getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     cThis = cLast;
 
                 else if (cLast != ' ' && cLast != '\n') {
 
-                    if (GetTexMode() == MODE_RESTRICTED_HORIZONTAL)
+                    if (getTexMode() == MODE_RESTRICTED_HORIZONTAL)
                         fprintRTF("\\~");
                     else
                         fprintRTF(" ");
@@ -265,7 +236,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
             case '\n':
                 tabcounter = 0;
 
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH) {
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {
 
                     cNext = getNonBlank();
                     ungetTexChar(cNext);
@@ -276,12 +247,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
                     if (cNext == '\n') {    /* new paragraph ... skip all ' ' and '\n' */
                         pending_new_paragraph = 2;
                         CmdEndParagraph(0);
-                        cNext = getNonBlank();
-                        ungetTexChar(cNext);
 
                     } else {    /* add a space if needed */
                         ungetTexChar(cNext);
-                        if (mode != MODE_VERTICAL && cLast != ' ')
+                        if (getTexMode() != MODE_VERTICAL && cLast != ' ')
                             fprintRTF(" ");
                     }
                 }
@@ -292,20 +261,12 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 cNext = getTexChar();
                 diagnostics(5, "Processing $, next char <%c>", cNext);
 
-                if (cNext == '$' && GetTexMode() != MODE_MATH)
+                if (cNext == '$' && getTexMode() != MODE_MATH)
                     CmdEquation(EQN_DOLLAR_DOLLAR | ON);
                 else {
                     ungetTexChar(cNext);
                     CmdEquation(EQN_DOLLAR | ON);
                 }
-
-                /* 
-                   Formulas need to close all Convert() operations when they end This works for \begin{equation} but
-                   not $$ since the BraceLevel and environments don't get pushed properly.  We do it explicitly here. */
-                /* 
-                   if (GetTexMode() == MODE_MATH || GetTexMode() == MODE_DISPLAYMATH) PushBrace(); else { ret =
-                   RecursionLevel - PopBrace(); if (ret > 0) { ret--; RecursionLevel--; diagnostics(5, "Exiting Convert 
-                   via Math ret = %d", ret); return; } } */
                 break;
 
             case '&':
@@ -314,15 +275,20 @@ globals: fTex, fRtf and all global flags for convert (see above)
                     break;
                 }
 
-                if (GetTexMode() == MODE_MATH || GetTexMode() == MODE_DISPLAYMATH) {    /* in eqnarray */
-                    fprintRTF("\\tab ");
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {    /* in eqnarray */
+                    if (g_processing_fields) fprintRTF("}}}{\\fldrslt }}");
+					fprintRTF("\\tab ");
+                    if (g_processing_fields) fprintRTF("{{\\field{\\*\\fldinst{ EQ ");
                     g_equation_column++;
                     break;
                 }
 
                 if (g_processing_tabular) { /* in tabular */
+                	diagnostics(0,"this should not happen tabular should handle this!");
+                	/*
                     actCol++;
                     fprintRTF("\\cell\\pard\\intbl\\q%c ", colFmt[actCol]);
+                    */
                     break;
                 }
                 fprintRTF("&");
@@ -341,10 +307,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '-':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
-                    fprintRTF("-");
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
+                	CmdSymbolChar(0x2d);
                 else {
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
 
                     count = getSameChar('-') + 1;
 
@@ -361,17 +327,17 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '|':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     fprintRTF("|");
                 else
                     fprintRTF("\\emdash ");
                 break;
 
             case '\'':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     fprintRTF("'");
                 else {
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
                     count = getSameChar('\'') + 1;
                     if (count == 2)
                         fprintRTF("\\rdblquote ");
@@ -382,7 +348,8 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '`':
-                SetTexMode(MODE_HORIZONTAL);
+                if (getTexMode() == MODE_VERTICAL)
+                    changeTexMode(MODE_HORIZONTAL);
                 count = getSameChar('`') + 1;
                 if (count == 2)
                     fprintRTF("\\ldblquote ");
@@ -392,7 +359,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '\"':
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
                 if (GermanMode)
                     TranslateGerman();
                 else
@@ -400,9 +367,9 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '<':
-                if (mode == MODE_VERTICAL)
-                    SetTexMode(MODE_HORIZONTAL);
-                if (GetTexMode() == MODE_HORIZONTAL) {
+                if (getTexMode() == MODE_VERTICAL)
+                    changeTexMode(MODE_HORIZONTAL);
+                if (getTexMode() == MODE_HORIZONTAL) {
                     cNext = getTexChar();
                     if (cNext == '<') {
                         if (FrenchMode) {   /* not quite right */
@@ -426,9 +393,9 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '>':
-                if (mode == MODE_VERTICAL)
-                    SetTexMode(MODE_HORIZONTAL);
-                if (GetTexMode() == MODE_HORIZONTAL) {
+                if (getTexMode() == MODE_VERTICAL)
+                    changeTexMode(MODE_HORIZONTAL);
+                if (getTexMode() == MODE_HORIZONTAL) {
                     cNext = getTexChar();
                     if (cNext == '>')
                         fprintRTF("\\'bb");
@@ -441,10 +408,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '!':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     fprintRTF("!");
                 else {
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
                     if ((cNext = getTexChar()) && cNext == '`') {
                         fprintRTF("\\'a1 ");
                     } else {
@@ -455,7 +422,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '?':
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
                 if ((cNext = getTexChar()) && cNext == '`') {
                     fprintRTF("\\'bf ");
                 } else {
@@ -465,10 +432,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case ':':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     fprintRTF(":");
                 else {
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
                     if (FrenchMode)
                         fprintRTF("\\~:");
                     else
@@ -477,10 +444,10 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '.':
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH)
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
                     fprintRTF(".");
                 else {
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
                     fprintRTF(".");
 
                     /* try to simulate double spaces after sentences */
@@ -507,14 +474,14 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case '(':
-                if (g_processing_fields && g_escape_parent)
+                if (g_processing_fields && g_escape_parens)
                     fprintRTF("\\\\(");
                 else
                     fprintRTF("(");
                 break;
 
             case ')':
-                if (g_processing_fields && g_escape_parent)
+                if (g_processing_fields && g_escape_parens)
                     fprintRTF("\\\\)");
                 else
                     fprintRTF(")");
@@ -530,14 +497,27 @@ globals: fTex, fRtf and all global flags for convert (see above)
                 break;
 
             case ',':
-                if (g_field_separator == ',' && g_processing_fields)
-                    fprintRTF("\\\\,");
-                else
-                    fprintRTF(",");
+                if (g_field_separator == ',' && g_processing_fields) {
+                	if (g_processing_arrays) {
+                	/* this is crazy, fields fail if \, is present in EQ array! */
+                	/* substitute semi-colon because it is least worst option   */
+						fprintRTF(";"); 
+                	} else {
+						fprintRTF("\\\\,"); 
+                	}
+                } else {
+                /* ,, is \quotedblbase nearly always */
+                    if ((cNext = getTexChar()) && cNext == ',') {
+                        fprintRTF("\\'84");
+                    } else {
+                        fprintRTF(",");
+                        ungetTexChar(cNext);
+                    }
+                }
                 break;
 
             default:
-                if (mode == MODE_MATH || mode == MODE_DISPLAYMATH) {
+                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {
                 	if (('a' <= cThis && cThis <= 'z') || ('A' <= cThis && cThis <= 'Z')) {
                     	if (CurrentFontSeries() == F_SERIES_BOLD)    /* do not italicize */
                         	fprintRTF("%c", cThis);
@@ -551,7 +531,7 @@ globals: fTex, fRtf and all global flags for convert (see above)
 
                 } else {
 
-                    SetTexMode(MODE_HORIZONTAL);
+                    changeTexMode(MODE_HORIZONTAL);
                     fprintRTF("%c", cThis);
                 }
                 break;
@@ -570,54 +550,59 @@ void TranslateCommand()
 purpose: The function is called on a backslash in input file and
 	 tries to call the command-function for the following command.
 returns: success or not
-globals: fTex, fRtf, command-functions have side effects or recursive calls;
-         global flags for convert
  ****************************************************************************/
 {
     char cCommand[MAXCOMMANDLEN];
     int i, mode;
-    int cThis;
-
+    int cThis,cNext;
 
     cThis = getTexChar();
-    mode = GetTexMode();
+    mode = getTexMode();
 
     diagnostics(5, "Beginning TranslateCommand() \\%c", cThis);
 
     switch (cThis) {
+        case 'a':
+            if (!g_processing_tabbing) break;
+            cNext = getTexChar();
+            if (cNext=='=' ) {CmdMacronChar(0); return;}
+            if (cNext=='\'') {CmdAcuteChar(0); return;}
+            if (cNext=='`' ) {CmdAcuteChar(0); return;}
+            ungetTexChar(cNext);
+            break;
         case '}':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("\\}");
             return;
         case '{':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("\\{");
             return;
         case '#':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("#");
             return;
         case '$':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("$");
             return;
         case '&':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("&");
             return;
         case '%':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("%%");
             return;
         case '_':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("_");
             return;
 
@@ -625,10 +610,17 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
             CmdSlashSlash(0);
             return;
 
+        case 0x0D:  /*handle any CRLF that might have snuck in*/
+			cNext = getTexChar();
+			if (cNext != 0x0A)
+				ungetTexChar(cNext);
+			/* fall through */
+
+        case '\t':  /* tabs should already be spaces */
         case ' ':
-        case '\n':
+        case 0x0A:
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF(" ");     /* ordinary interword space */
             skipSpaces();
             return;
@@ -637,17 +629,17 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
         case '-':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 PushBrace();
             } else
-                fprintRTF("\\-");
+				fprintRTF("\\-");
             return;
 
         case '+':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 PushBrace();
@@ -656,7 +648,7 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
         case '<':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 PushBrace();
@@ -665,7 +657,7 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
         case '>':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 CmdTabjump();
@@ -676,28 +668,28 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
         case '`':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 PushBrace();
             } else
-                CmdLApostrophChar(0);
+                CmdGraveChar(0);
             return;
 
         case '\'':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 PushBrace();
                 return;
             } else
-                CmdRApostrophChar(0);   /* char ' =?= \' */
+                CmdAcuteChar(0);   /* char ' =?= \' */
             return;
 
         case '=':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             if (g_processing_tabbing) {
                 (void) PopBrace();
                 CmdTabset();
@@ -708,15 +700,17 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
         case '~':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             CmdTildeChar(0);
             return;
+            
         case '^':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             
             cThis = getTexChar();
-            if (cThis=='^') {		/* replace \^^M with space */
+            if (cThis=='^') {
+            	/* usually \^^M ... just replace with a blank */
             	cThis = getTexChar();
             	fprintRTF(" ");
             } else {
@@ -726,12 +720,12 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
             return;
         case '.':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             CmdDotChar(0);
             return;
         case '\"':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             CmdUmlauteChar(0);
             return;
         case '(':
@@ -758,12 +752,12 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
             return;
         case ',':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
-            CmdSpace(0.33);     /* \, produces a small space */
+                changeTexMode(MODE_HORIZONTAL);
+            CmdNonBreakSpace(50);     /* \, produces a small space = 50% */
             return;
         case ';':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             CmdSpace(0.75);     /* \; produces a thick space */
             return;
         case '@':
@@ -771,13 +765,13 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
             return;
         case '3':
             if (mode == MODE_VERTICAL)
-                SetTexMode(MODE_HORIZONTAL);
+                changeTexMode(MODE_HORIZONTAL);
             fprintRTF("{\\'df}");   /* german symbol '' */
             return;
     }
 
 
-    /* LEG180498 Commands consist of letters and can have an optional * at the end */
+    /* Commands consist of letters and can have an optional * at the end */
     for (i = 0; i < MAXCOMMANDLEN-1; i++) {
         if (!isalpha((int) cThis) && (cThis != '*')) {
             bool found_nl = FALSE;
@@ -820,7 +814,9 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
 
     if (CallCommandFunc(cCommand)) {    /* call handling function for command */
         if (strcmp(cCommand, "end") == 0) {
+    		diagnostics(5, "before PopBrace()");
             ret = RecursionLevel - PopBrace();
+    		diagnostics(5, "after PopBrace(), ret=%d",ret);
             fprintRTF("}");
         }
         return;
@@ -832,5 +828,5 @@ globals: fTex, fRtf, command-functions have side effects or recursive calls;
     if (TryVariableIgnore(cCommand))
         return;
 
-    diagnostics(WARNING, "Command \\%s not found - ignored", cCommand);
+    diagnostics(WARNING, "Unknown command '\\%s'", cCommand);
 }
