@@ -22,19 +22,20 @@ This file is available from http://sourceforge.net/projects/latex2rtf/
 
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include "main.h"
 #include "convert.h"
 #include "definitions.h"
 #include "parser.h"
 #include "funct1.h"
-#include "util.h"
+#include "utils.h"
 #include "cfg.h"
 #include "counters.h"
 #include "funct1.h"
 
-#define MAX_DEFINITIONS 200
-#define MAX_ENVIRONMENTS 20
-#define MAX_THEOREMS 20
+#define MAX_DEFINITIONS 2000
+#define MAX_ENVIRONMENTS 200
+#define MAX_THEOREMS 200
 
 struct {
     char *name;
@@ -83,12 +84,12 @@ static int strequal(char *a, char *b)
 /* static void printDefinitions(void)
 {
 int i=0;
-	fprintf(stderr, "\n");
+	diagnostics(WARNING, "\n");
 	while(i < iDefinitionCount ) {
-		fprintf(stderr, "[%d] name     =<%s>\n",i, Definitions[i].name);
-		fprintf(stderr, "    opt_param=<%s>\n", Definitions[i].opt_param);
-		fprintf(stderr, "    def      =<%s>\n", Definitions[i].def);
-		fprintf(stderr, "    params   =<%d>\n", Definitions[i].params);
+		diagnostics(WARNING, "[%d] name     =<%s>\n",i, Definitions[i].name);
+		diagnostics(WARNING, "    opt_param=<%s>\n", Definitions[i].opt_param);
+		diagnostics(WARNING, "    def      =<%s>\n", Definitions[i].def);
+		diagnostics(WARNING, "    params   =<%d>\n", Definitions[i].params);
 		i++;
 	}
 }
@@ -96,12 +97,12 @@ int i=0;
 static void printTheorems(void)
 {
 int i=0;
-	fprintf(stderr, "\n");
+	diagnostics(WARNING, "\n");
 	for (i=0; i< iNewTheoremCount; i++) {
-		fprintf(stderr, "[%d] name   =<%s>\n",i, NewTheorems[i].name);
-		fprintf(stderr, "    caption    =<%s>\n", NewTheorems[i].caption);
-		fprintf(stderr, "    like =<%s>\n", NewTheorems[i].numbered_like);
-		fprintf(stderr, "    within    =<%s>\n", NewTheorems[i].within);
+		diagnostics(WARNING, "[%d] name   =<%s>\n",i, NewTheorems[i].name);
+		diagnostics(WARNING, "    caption    =<%s>\n", NewTheorems[i].caption);
+		diagnostics(WARNING, "    like =<%s>\n", NewTheorems[i].numbered_like);
+		diagnostics(WARNING, "    within    =<%s>\n", NewTheorems[i].within);
 	}
 }
 */
@@ -129,9 +130,9 @@ static char *expandmacro(char *macro, char *opt_param, int params)
     }
 
     for (; i < params; i++) {
-        args[i] = getBraceParam();
+        args[i] = getBraceRawParam();
         buff_size += strlen(args[i]);
-        diagnostics(3, "argument #%d <%s>", i + 1, args[i]);
+        diagnostics(5, "argument #%d <%s>", i + 1, args[i]);
     }
 
     dmacro = strdup(macro);
@@ -181,7 +182,12 @@ static char *expandmacro(char *macro, char *opt_param, int params)
 
             } else if (param < params) {
                 diagnostics(3, "expandmacro arg =<%s>", args[param]);
-                if (expanded+strlen(args[param]) <buffer+buff_size) {
+                if (expanded+strlen(args[param])+1 <buffer+buff_size) {
+                    char c = *(expanded-1);
+                    if (isalpha(c)) {
+                    	*expanded=' ';
+                        expanded++;
+                    }
                     strcpy(expanded, args[param]);
                     expanded += strlen(args[param]);
                 } else 
@@ -225,7 +231,11 @@ int maybeDefinition(char *s, size_t n)
         return TRUE;
 
     for (i = 0; i < iDefinitionCount; i++) {
-        diagnostics(6, "seeking=<%s>, i=%d, current=<%s>", s, i, Definitions[i].name);
+    	/*
+    	char *ss = my_strndup(s,strlen(Definitions[i].name));
+        diagnostics(WARNING, "def seeking=<%s>, i=%d, current=<%s>", ss, i, Definitions[i].name);
+        free(ss);
+        */
         if (strncmp(s, Definitions[i].name, n) == 0)
             return TRUE;
     }
@@ -243,9 +253,19 @@ int existsDefinition(char *s)
     int i;
 
     for (i = 0; i < iDefinitionCount; i++) {
-        diagnostics(6, "seeking=<%s>, i=%d, current=<%s>", s, i, Definitions[i].name);
-        if (strcmp(s, Definitions[i].name) == 0)
+    /*
+		char *tt = my_strndup(s,strlen(Definitions[i].name));
+		diagnostics(WARNING, "def text has=<%s>, i=%d, trying=<%s>", tt, i, Definitions[i].name);
+		free(tt);
+	*/
+        if (strcmp(s, Definitions[i].name) == 0) {
+    /*
+			char *ss = my_strndup(s,strlen(Definitions[i].name));
+			diagnostics(WARNING, "def text has=<%s>, i=%d, matched=<%s>", ss, i, Definitions[i].name);
+			free(ss);
+	*/
             break;
+        }
     }
 
     if (i == iDefinitionCount)
@@ -262,7 +282,7 @@ void newDefinition(char *name, char *opt_param, char *def, int params)
               define \hd, name = "hd"
 **************************************************************************/
 {
-    diagnostics(3, "Adding macro <%s>=<%s>", name, def);
+    diagnostics(2, "Adding macro '%s' = [%s]", name, def);
 
     if (strcmp(name, "LaTeX") == 0)
         return;
@@ -318,12 +338,12 @@ void renewDefinition(char *name, char *opt_param, char *def, int params)
 {
     int i;
 
-    diagnostics(3, "renewDefinition seeking <%s>\n", name);
+    diagnostics(3, "renewDefinition seeking '%s'\n", name);
     i = existsDefinition(name);
 
     if (i < 0) {
         newDefinition(name, opt_param, def, params);
-        diagnostics(WARNING, "No existing definition for \\%s", name);
+        diagnostics(2, "No existing definition for \\%s", name);
 
     } else {
         free(Definitions[i].def);
@@ -378,14 +398,16 @@ int existsEnvironment(char *s)
 
     n = strlen(s);
     while (i < iNewEnvironmentCount && !strequal(s, NewEnvironments[i].name)) {
-        diagnostics(4, "e seeking=<%s>, i=%d, current=<%s>", s, i, NewEnvironments[i].name);
+        diagnostics(6, "existsEnv seeking=<%s>, i=%d, current=<%s>", s, i, NewEnvironments[i].name);
         i++;
     }
 
     if (i == iNewEnvironmentCount)
         return -1;
-    else
+    else {
+        diagnostics(4, "user env found=<%s>, i=%d, current=<%s>", s, i, NewEnvironments[i].name);
         return i;
+    }
 }
 
 int maybeEnvironment(char *s, size_t n)
@@ -401,14 +423,38 @@ int maybeEnvironment(char *s, size_t n)
         return TRUE;
 
     for (i = 0; i < iNewEnvironmentCount; i++) {
-        diagnostics(6, "seeking=<%s>, i=%d, current=<%s>", s, i, NewEnvironments[i].name);
-        if (strncmp(s, NewEnvironments[i].begname, n) == 0 || strncmp(s, NewEnvironments[i].endname, n) == 0) {
-            diagnostics(6, "possible");
+    
+		if (0) {
+		char *ss = my_strndup(s,n);
+		diagnostics(2, "trying env seeking=<%s>, i=%d, current=<%s>", ss, i, NewEnvironments[i].endname);
+		free(ss);
+		}
+		
+        if (strncmp(s, NewEnvironments[i].begname, n) == 0) {
+			if (0) {
+			char *ss = my_strndup(s,n);
+			diagnostics(2, "possible env seeking=<%s>, i=%d, current=<%s>", ss, i, NewEnvironments[i].begname);
+			free(ss);
+			}
+            return TRUE;
+        }
+
+    
+        if (strncmp(s, NewEnvironments[i].endname, n) == 0) {
+			if (0) {
+			char *ss = my_strndup(s,n);
+			diagnostics(2, "possible env seeking=<%s>, i=%d, current=<%s>", ss, i, NewEnvironments[i].endname);
+			free(ss);
+			}
             return TRUE;
         }
     }
 
-    diagnostics(6, "not possible");
+	if (0) {
+	char *ss = my_strndup(s,n);
+	diagnostics(2, "failed all env seeking=<%s>", ss);
+	free(ss);
+	}
     return FALSE;
 }
 
@@ -464,7 +510,7 @@ void renewEnvironment(char *name, char *opt_param, char *begdef, char *enddef, i
 
     if (i < 0) {
         newEnvironment(name, opt_param, begdef, enddef, params);
-        diagnostics(WARNING, "No existing \\newevironment{%s}", name);
+        diagnostics(2, "No existing \\newevironment{%s}", name);
 
     } else {
         free(NewEnvironments[i].begdef);
@@ -496,20 +542,27 @@ char *expandEnvironment(int thedef, int code)
      purpose: retrieves and expands a \newenvironment 
 **************************************************************************/
 {
+    char *s, *t;
+    
     if (thedef < 0 || thedef >= iNewEnvironmentCount)
         return NULL;
 
     if (code == CMD_BEGIN) {
 
         diagnostics(3, "\\begin{%s} <%s>", NewEnvironments[thedef].name, NewEnvironments[thedef].begdef);
-        return expandmacro(NewEnvironments[thedef].begdef,
+        s= expandmacro(NewEnvironments[thedef].begdef,
           NewEnvironments[thedef].opt_param, NewEnvironments[thedef].params);
+        t = strdup_together("{",s);
 
     } else {
 
         diagnostics(3, "\\end{%s} <%s>", NewEnvironments[thedef].name, NewEnvironments[thedef].enddef);
-        return expandmacro(NewEnvironments[thedef].enddef, NULL, 0);
+        s = expandmacro(NewEnvironments[thedef].enddef, NULL, 0);
+        t = strdup_together(s,"}");
     }
+
+    free(s);
+    return t;
 }
 
 void newTheorem(char *name, char *caption, char *numbered_like, char *within)
