@@ -28,7 +28,7 @@ By convention all the values stored should be twips  20 twips = 1 pt
 #include <stdlib.h>
 #include <string.h>
 #include "main.h"
-#include "util.h"
+#include "utils.h"
 #include "lengths.h"
 #include "parser.h"
 
@@ -74,8 +74,7 @@ static void newLength(char *s, int d)
     Lengths[iLengthCount].name = strdup(s);
 
     if (Lengths[iLengthCount].name == NULL) {
-        fprintf(stderr, "\nCannot allocate name for length \\%s\n", s);
-        exit(1);
+        diagnostics(ERROR, "Cannot allocate name for length \\%s", s);
     }
 
     iLengthCount++;
