@@ -1,7 +1,7 @@
 
 /* main.c - LaTeX to RTF conversion program
 
-Copyright (C) 1995-2002 The Free Software Foundation
+Copyright (C) 1995-2007 The Free Software Foundation
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,7 +23,7 @@ Authors:
 	1995	  Fernando Dorner, Andreas Granzer, Freidrich Polzer, Gerhard Trisko
 	1995-1997 Ralf Schlatterbeck
 	1998-2000 Georg Lehner
-	2001-2002 Scott Prahl
+	2001-2007 Scott Prahl
 */
 
 #include <stdio.h>
@@ -37,20 +37,22 @@ Authors:
 #include "convert.h"
 #include "commands.h"
 #include "chars.h"
-#include "l2r_fonts.h"
+#include "fonts.h"
 #include "stack.h"
 #include "direct.h"
 #include "ignore.h"
 #include "version.h"
 #include "funct1.h"
 #include "cfg.h"
-#include "encode.h"
-#include "util.h"
+#include "encodings.h"
+#include "utils.h"
 #include "parser.h"
 #include "lengths.h"
 #include "counters.h"
 #include "preamble.h"
-#include "xref.h"
+#include "xrefs.h"
+#include "preparse.h"
+#include "vertical.h"
 
 FILE *fRtf = NULL;              /* file pointer to RTF file */
 char *g_tex_name = NULL;
@@ -60,12 +62,10 @@ char *g_toc_name = NULL;
 char *g_lof_name = NULL;
 char *g_lot_name = NULL;
 char *g_fff_name = NULL;
+char *g_ttt_name = NULL;
 char *g_bbl_name = NULL;
 char *g_home_dir = NULL;
 
-/*** interpret comment lines that follow the '%' with this string ***/
-const char *InterpretCommentString = "latex2rtf:";
-
 char *progname;                 /* name of the executable file */
 bool GermanMode = FALSE;        /* support germanstyle */
 bool FrenchMode = FALSE;        /* support frenchstyle */
@@ -93,6 +93,7 @@ bool g_show_equation_number = FALSE;
 int g_enumerate_depth = 0;
 bool g_suppress_equation_number = FALSE;
 bool g_aux_file_missing = FALSE;    /* assume that it exists */
+bool g_bbl_file_missing = FALSE;    /* assume that it exists */
 
 bool g_document_type = FORMAT_ARTICLE;
 int g_document_bibstyle = BIBSTYLE_STANDARD;
@@ -104,21 +105,30 @@ int g_safety_braces = 0;
 bool g_processing_equation = FALSE;
 bool g_RTF_warnings = FALSE;
 char *g_config_path = NULL;
-char *g_script_path = NULL;
-char *g_tmp_path = NULL;
+char *g_script_dir = NULL;
+char *g_tmp_dir = NULL;
 char *g_preamble = NULL;
 char g_field_separator = ',';
-bool g_escape_parent = TRUE;
+bool g_escape_parens = FALSE;
 
 bool g_equation_display_rtf = TRUE;
 bool g_equation_inline_rtf = TRUE;
 bool g_equation_inline_bitmap = FALSE;
 bool g_equation_display_bitmap = FALSE;
 bool g_equation_comment = FALSE;
+bool g_equation_raw_latex = FALSE;
+bool g_tableofcontents = FALSE;
+
+bool g_tabular_display_rtf = TRUE;
+bool g_tabular_display_bitmap = FALSE;
 
 double g_png_equation_scale = 1.22;
 double g_png_figure_scale = 1.35;
 bool g_latex_figures = FALSE;
+bool g_endfloat_figures = FALSE;
+bool g_endfloat_tables = FALSE;
+bool g_endfloat_markers = TRUE;
+int  g_graphics_package = GRAPHICS_NONE;
 
 int indent = 0;
 char alignment = JUSTIFIED;     /* default for justified: */
@@ -154,7 +164,7 @@ int main(int argc, char **argv)
     InitializeLatexLengths();
 	InitializeBibliography();
 	
-    while ((c = my_getopt(argc, argv, "lhpvFSWZ:o:a:b:d:f:i:s:C:D:M:P:T:")) != EOF) {
+    while ((c = my_getopt(argc, argv, "lhpuvFSVWZ:o:a:b:d:f:i:s:u:C:D:M:P:T:t:")) != EOF) {
         switch (c) {
             case 'a':
                 g_aux_name = optarg;
@@ -171,7 +181,6 @@ int main(int argc, char **argv)
                 break;
             case 'f':
                 sscanf(optarg, "%d", &x);
-                diagnostics(2, "Field option = %s x=%d", optarg, x);
                 g_fields_use_EQ = (x & 1) ? TRUE : FALSE;
                 g_fields_use_REF = (x & 2) ? TRUE : FALSE;
                 break;
@@ -185,7 +194,7 @@ int main(int argc, char **argv)
                 g_rtf_name = strdup(optarg);
                 break;
             case 'p':
-                g_escape_parent = FALSE;
+                g_escape_parens = TRUE;
                 break;
             case 'v':
                 print_version();
@@ -196,60 +205,83 @@ int main(int argc, char **argv)
             case 'D':
                 sscanf(optarg, "%d", &g_dots_per_inch);
                 if (g_dots_per_inch < 25 || g_dots_per_inch > 600)
-                    fprintf(stderr, "Dots per inch must be between 25 and 600 dpi\n");
+                    diagnostics(WARNING, "Dots per inch must be between 25 and 600 dpi\n");
                 break;
             case 'F':
                 g_latex_figures = TRUE;
                 break;
             case 'M':
                 sscanf(optarg, "%d", &x);
-                diagnostics(2, "Math option = %s x=%d", optarg, x);
-                g_equation_display_rtf = (x & 1) ? TRUE : FALSE;
-                g_equation_inline_rtf = (x & 2) ? TRUE : FALSE;
-                g_equation_display_bitmap = (x & 4) ? TRUE : FALSE;
-                g_equation_inline_bitmap = (x & 8) ? TRUE : FALSE;
-                g_equation_comment = (x & 16) ? TRUE : FALSE;
-                diagnostics(2, "Math option g_equation_display_rtf      = %d", g_equation_display_rtf);
-                diagnostics(2, "Math option g_equation_inline_rtf	      = %d", g_equation_inline_rtf);
-                diagnostics(2, "Math option g_equation_display_bitmap   = %d", g_equation_display_bitmap);
-                diagnostics(2, "Math option g_equation_inline_bitmap    = %d", g_equation_inline_bitmap);
-                diagnostics(2, "Math option g_equation_comment	      = %d", g_equation_comment);
-                if (!g_equation_comment && !g_equation_inline_rtf && !g_equation_inline_bitmap)
+                diagnostics(3, "Math option = %s x=%d", optarg, x);
+                g_equation_display_rtf   = (x &  1) ? TRUE : FALSE;
+                g_equation_inline_rtf    = (x &  2) ? TRUE : FALSE;
+                g_equation_display_bitmap= (x &  4) ? TRUE : FALSE;
+                g_equation_inline_bitmap = (x &  8) ? TRUE : FALSE;
+                g_equation_comment       = (x & 16) ? TRUE : FALSE;
+                g_equation_raw_latex     = (x & 32) ? TRUE : FALSE;
+                diagnostics(3, "Math option g_equation_display_rtf    = %d", g_equation_display_rtf);
+                diagnostics(3, "Math option g_equation_inline_rtf     = %d", g_equation_inline_rtf);
+                diagnostics(3, "Math option g_equation_display_bitmap = %d", g_equation_display_bitmap);
+                diagnostics(3, "Math option g_equation_inline_bitmap  = %d", g_equation_inline_bitmap);
+                diagnostics(3, "Math option g_equation_comment        = %d", g_equation_comment);
+                diagnostics(3, "Math option g_equation_raw_latex      = %d", g_equation_raw_latex);
+                if (!g_equation_comment && !g_equation_inline_rtf && !g_equation_inline_bitmap && !g_equation_raw_latex)
                     g_equation_inline_rtf = TRUE;
-                if (!g_equation_comment && !g_equation_display_rtf && !g_equation_display_bitmap)
+                if (!g_equation_comment && !g_equation_display_rtf && !g_equation_display_bitmap && !g_equation_raw_latex)
                     g_equation_display_rtf = TRUE;
                 break;
 
+            case 't':
+                sscanf(optarg, "%d", &x);
+                diagnostics(3, "Table option = %s x=%d", optarg, x);
+                g_tabular_display_rtf    = (x &  1) ? TRUE : FALSE;
+                g_tabular_display_bitmap = (x &  2) ? TRUE : FALSE;
+                diagnostics(3, "Table option g_tabular_display_rtf     = %d", g_tabular_display_rtf);
+                diagnostics(3, "Table option g_tabular_display_bitmap  = %d", g_tabular_display_bitmap);
+                break;
+
             case 'P':          /* -P path/to/cfg:path/to/script or -P path/to/cfg or -P :path/to/script */
-                p = strchr(optarg, ':');
+                p = strchr(optarg, ENVSEP);
                 if (p) {
                     *p = '\0';
-                    g_script_path = strdup(p + 1);
+                    g_script_dir = strdup(p + 1);
                 }
                 if (p != optarg)
                     g_config_path = strdup(optarg);
-                diagnostics(2, "cfg=%s, script=%s", g_config_path, g_script_path);
+                diagnostics(2, "cfg=%s, script=%s", g_config_path, g_script_dir);
                 break;
+
             case 's':
                 if (optarg && optarg[0] == 'e') {
-                    if (sscanf(optarg, "e%lf", &xx) == 1 && xx > 0)
+                    if (sscanf(optarg, "e%lf", &xx) == 1 && xx > 0) {
                         g_png_equation_scale = xx;
-                    else
-                        diagnostics(WARNING, "Equation scale (-se #) is not positive, ignoring");
-                }
-                if (optarg && optarg[0] == 'f') {
-                    if (sscanf(optarg, "f%lf", &xx) == 1 && xx > 0)
+                    } else {
+                        diagnostics(WARNING, "Mistake in command line number for scaling equations");
+                        diagnostics(WARNING, "Either use no spaces: '-se1.22' or write as '-s e1.22'");
+                    }
+                    
+                } else if (optarg && optarg[0] == 'f') {
+                
+                    if (sscanf(optarg, "f%lf", &xx) == 1 && xx > 0) {
                         g_png_figure_scale = xx;
-                    else
-                        diagnostics(WARNING, "Figure scale (-sf #) is not positive, ignoring");
+                    } else {
+                        diagnostics(WARNING, "Mistake in command line number for scaling figures");
+                        diagnostics(WARNING, "Either use no spaces: '-sf1.35' or write as '-s f1.35'");
+                    }
+                } else {
+                    diagnostics(WARNING, "Unknown option '-s' use '-se#' or '-sf#'");
                 }
                 break;
+
             case 'S':
                 g_field_separator = ';';
                 break;
             case 'T':
-                g_tmp_path = strdup(optarg);
+                g_tmp_dir = strdup(optarg);
                 break;
+            case 'V':
+                print_version();
+                return (0);
             case 'W':
                 g_RTF_warnings = TRUE;
                 break;
@@ -329,12 +361,15 @@ int main(int argc, char **argv)
     if (g_fff_name == NULL && basename != NULL)
         g_fff_name = strdup_together(basename, ".fff");
 
+    if (g_ttt_name == NULL && basename != NULL)
+        g_ttt_name = strdup_together(basename, ".ttt");
+
     if (basename) {
-        diagnostics(3, "latex filename is <%s>", g_tex_name);
-        diagnostics(3, "  rtf filename is <%s>", g_rtf_name);
-        diagnostics(3, "  aux filename is <%s>", g_aux_name);
-        diagnostics(3, "  bbl filename is <%s>", g_bbl_name);
-        diagnostics(3, "home directory is <%s>", (g_home_dir) ? g_home_dir : "");
+        diagnostics(2, "latex filename is <%s>", g_tex_name);
+        diagnostics(2, "  rtf filename is <%s>", g_rtf_name);
+        diagnostics(2, "  aux filename is <%s>", g_aux_name);
+        diagnostics(2, "  bbl filename is <%s>", g_bbl_name);
+        diagnostics(2, "home directory is <%s>", (g_home_dir) ? g_home_dir : "");
     }
 
     ReadCfg();
@@ -350,7 +385,9 @@ int main(int argc, char **argv)
         CloseRtf(&fRtf);
         printf("\n");
 
-/*		debug_malloc();*/
+/*		debug malloc() 
+  		printf("Done!");
+  		while (1) {} do nothing */
 
         return 0;
     } else {
@@ -378,50 +415,75 @@ static void ConvertWholeDocument(void)
     char *body, *sec_head, *sec_head2, *label;
     char t[] = "\\begin{document}";
 
-    PushEnvironment(DOCUMENT);  /* because we use ConvertString in preamble.c */
-    PushEnvironment(PREAMBLE);
-    SetTexMode(MODE_VERTICAL);
+    PushEnvironment(DOCUMENT_MODE);  /* because we use ConvertString in preamble.c */
+    PushEnvironment(PREAMBLE_MODE);
+    setTexMode(MODE_VERTICAL);
     ConvertLatexPreamble();
     WriteRtfHeader();
     ConvertString(t);
 
     g_processing_preamble = FALSE;
-    getSection(&body, &sec_head, &label);
+    preParse(&body, &sec_head, &label);
 
-    diagnostics(2, "*******************\nbody=%s", (body) ? body : "<empty>");
-    diagnostics(2, "*******************\nsec_head=%s", (sec_head) ? sec_head : "<none>");
-    diagnostics(2, "*******************\nlabel=%s", (g_section_label) ? g_section_label : "<none>");
+    diagnostics(2, "\\begin{document}");
+	diagnostics(5,"label for this section is'%s'", label);
+	diagnostics(5, "next section '%s'", sec_head);
+	show_string(2, body, "body ");
+	
     ConvertString(body);
     free(body);
     if (label)
         free(label);
 
-    while (sec_head) {
-        getSection(&body, &sec_head2, &g_section_label);
+    while (strcmp(sec_head,"\\end{document}")!=0) {
+        preParse(&body, &sec_head2, &g_section_label);
         label = ExtractLabelTag(sec_head);
         if (label) {
             if (g_section_label)
                 free(g_section_label);
             g_section_label = label;
         }
-        diagnostics(2, "\n========this section head==========\n%s", (sec_head) ? sec_head : "<none>");
-        diagnostics(2, "\n============ label ================\nlabel=%s",
-          (g_section_label) ? g_section_label : "<none>");
-        diagnostics(2,
-          "\n==============body=================\n%s\n=========end	body=================", (body) ? body : "<empty>");
-        diagnostics(2, "\n========next section head==========\n%s", (sec_head2) ? sec_head2 : "<none>");
+        
+		diagnostics(2, "processing '%s'", sec_head);	
+		diagnostics(5, "label is   '%s'", g_section_label);
+		diagnostics(5, "next  is   '%s'", sec_head2);	
+		show_string(2, body, "body ");
+
         ConvertString(sec_head);
         ConvertString(body);
         free(body);
         free(sec_head);
         sec_head = sec_head2;
     }
+    
+    if (g_endfloat_figures && g_fff_name) {
+    	g_endfloat_figures = FALSE;
+        if (PushSource(g_fff_name, NULL) == 0) {
+        	CmdNewPage(NewPage);
+        	CmdListOf(LIST_OF_FIGURES);
+        	Convert();
+        }
+     }
+    
+    if (g_endfloat_tables && g_ttt_name) {
+    	g_endfloat_tables = FALSE;
+        if (PushSource(g_ttt_name, NULL) == 0) {
+        	CmdNewPage(NewPage);
+        	CmdListOf(LIST_OF_TABLES);
+        	Convert();
+        }
+     }
+     
+    if (strcmp(sec_head,"\\end{document}")==0) {
+        diagnostics(2, "\\end{document}");
+    	ConvertString(sec_head);
+	}
 }
 
 static void print_version(void)
 {
     fprintf(stdout, "latex2rtf %s\n\n", Version);
-    fprintf(stdout, "Copyright (C) 2002 Free Software Foundation, Inc.\n");
+    fprintf(stdout, "Copyright (C) 2007 Free Software Foundation, Inc.\n");
     fprintf(stdout, "This is free software; see the source for copying conditions.  There is NO\n");
     fprintf(stdout, "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
     fprintf(stdout, "Written by Prahl, Lehner, Granzer, Dorner, Polzer, Trisko, Schlatterbeck.\n");
@@ -459,12 +521,16 @@ static void print_usage(void)
     fprintf(stdout, "       -M8          inline equations to bitmap\n");
     fprintf(stdout, "       -M12         inline and displayed equations to bitmaps\n");
     fprintf(stdout, "       -M16         insert Word comment field that the original equation text\n");
+    fprintf(stdout, "       -M32         insert the raw LaTeX equation delimited by <<: and :>>\n");
     fprintf(stdout, "  -o outputfile    file for RTF output\n");
     fprintf(stdout, "  -p               option to avoid bug in Word for some equations\n");
     fprintf(stdout, "  -P path          paths to *.cfg & latex2png\n");
     fprintf(stdout, "  -S               use ';' to separate args in RTF fields\n");
     fprintf(stdout, "  -se#             scale factor for bitmap equations\n");
     fprintf(stdout, "  -sf#             scale factor for bitmap figures\n");
+    fprintf(stdout, "  -t#              table handling\n");
+    fprintf(stdout, "       -t1          tabular and tabbing environments as RTF\n");
+    fprintf(stdout, "       -t2          tabular and tabbing environments as bitmaps\n");
     fprintf(stdout, "  -T /path/to/tmp  temporary directory\n");
     fprintf(stdout, "  -v               version information\n");
     fprintf(stdout, "  -V               version information\n");
@@ -475,6 +541,7 @@ static void print_usage(void)
     fprintf(stdout, "  latex2rtf <foo >foo.RTF             convert foo to foo.RTF\n");
     fprintf(stdout, "  latex2rtf -P ./cfg/:./scripts/ foo  use alternate cfg and latex2png files\n");
     fprintf(stdout, "  latex2rtf -M12 foo                  replace equations with bitmaps\n");
+    fprintf(stdout, "  latex2rtf -t3  foo                  tables as RTF *and* bitmaps\n");
     fprintf(stdout, "  latex2rtf -i russian foo            assume russian tex conventions\n");
     fprintf(stdout, "  latex2rtf -C raw foo                retain font encoding in rtf file\n");
     fprintf(stdout, "  latex2rtf -f0 foo                   create foo.rtf without fields\n");
@@ -496,6 +563,8 @@ void diagnostics(int level, char *format, ...)
 purpose: Writes the message to stderr depending on debugging level
  ****************************************************************************/
 {
+    static int first = TRUE;
+    
     char buffer[512], *buff_ptr;
     va_list apf;
     int i, iEnvCount;
@@ -508,7 +577,9 @@ purpose: Writes the message to stderr depending on debugging level
 
         iEnvCount = CurrentEnvironmentCount();
 
-        fprintf(stderr, "\n%s:%d ",CurrentFileName(),CurrentLineNumber());
+        if (!first) fprintf(stderr,"\n");
+        
+        fprintf(stderr, "%s:%-3d ",CurrentFileName(),CurrentLineNumber());
         switch (level) {
             case 0:
                 fprintf(stderr, "Error! ");
@@ -518,7 +589,7 @@ purpose: Writes the message to stderr depending on debugging level
                     vsnprintf(buffer, 512, format, apf);
                     fprintRTF("{\\plain\\cf2 [latex2rtf:");
                     while (*buff_ptr) {
-                        putRtfChar(*buff_ptr);
+                        putRtfCharEscaped(*buff_ptr);
                         buff_ptr++;
                     }
                     fprintRTF("]}");
@@ -543,6 +614,7 @@ purpose: Writes the message to stderr depending on debugging level
                 break;
         }
         vfprintf(stderr, format, apf);
+    	first = FALSE;
     }
     va_end(apf);
 
@@ -593,6 +665,8 @@ static void InitializeLatexLengths(void)
     setCounter("footnote", 0);
     setCounter("mpfootnote", 0);
     setCounter("secnumdepth", 2);
+    setCounter("endfloatfigure", 0);
+    setCounter("endfloattable", 0);
 
 /* vertical separation lengths */
     setLength("topsep", 3 * 20);
@@ -620,45 +694,33 @@ static void InitializeLatexLengths(void)
     setLength("marginparsep", 10 * 20);
 }
 
-/****************************************************************************
-purpose: removes %InterpretCommentString from preamble (usually "%latex2rtf:")
- ****************************************************************************/
-static void RemoveInterpretCommentString(char *s)
-{
-    char *p, *t;
-    int n = strlen(InterpretCommentString);
-
-    t = s;
-    while ((p = strstr(t, InterpretCommentString))) {
-        t = p - 1;
-        if (*t == '%')
-            strcpy(t, t + n + 1);
-        else
-            t += n + 1;
-    }
-}
-
 static void ConvertLatexPreamble(void)
 
 /****************************************************************************
 purpose: reads the LaTeX preamble (to \begin{document} ) for the file
  ****************************************************************************/
 {
-    FILE *hidden;
-    char t[] = "\\begin{document}";
+    char t[] = "\\begin|{|document|}";
+    FILE *rtf_file;
 
-    diagnostics(4, "Reading LaTeX Preamble");
-    hidden = fRtf;
-    fRtf = stderr;
+	/* Here we switch the file pointers ... it is important that nothing
+	   get printed to fRtf until the entire preamble has been processed.  
+	   This is really hard to track down, so the processed RTF get sent
+	   directly to stderr instead.
+	*/
+	rtf_file = fRtf;
+	fRtf = stderr;
+	
+    g_preamble = getSpacedTexUntil(t, 1);
 
-    g_preamble = getTexUntil(t, 1);
-    RemoveInterpretCommentString(g_preamble);
+    diagnostics(2, "Read LaTeX Preamble");
+    diagnostics(5, "Entering ConvertString() from ConvertLatexPreamble");
+
+	show_string(5, g_preamble, "preamble");	
 
-    diagnostics(4, "Entering ConvertString() from ConvertLatexPreamble <%s>", g_preamble);
     ConvertString(g_preamble);
-    diagnostics(4, "Exiting ConvertString() from ConvertLatexPreamble");
-
-    fRtf = hidden;
+    diagnostics(5, "Exiting ConvertString() from ConvertLatexPreamble");
+    fRtf = rtf_file;
 }
 
 
@@ -683,12 +745,12 @@ params: filename - name of outputfile, possibly NULL for already open file
         else
             name = strdup(filename);
 
-        diagnostics(3, "Opening RTF file <%s>", name);
         *f = fopen(name, "w");
 
         if (*f == NULL)
             diagnostics(ERROR, "Error opening RTF file <%s>\n", name);
 
+        diagnostics(2, "Opened RTF file <%s>", name);
         free(name);
     }
 }
@@ -704,9 +766,14 @@ globals: g_tex_name;
     int i;
 
     CmdEndParagraph(0);
-    if (BraceLevel > 1)
+    if (BraceLevel > 1) {
         diagnostics(WARNING, "Mismatched '{' in RTF file, Conversion may cause problems.");
-
+        diagnostics(WARNING, "This is often caused by having environments that span ");
+        diagnostics(WARNING, "\\section{}s.  For example ");
+        diagnostics(WARNING, "   \\begin{small} ... \\section{A} ... \\section{B} ... \\end{small}");
+        diagnostics(WARNING, "will definitely fail.");
+	}
+	
     if (BraceLevel - 1 > g_safety_braces)
         diagnostics(WARNING, "Try translating with 'latex2rtf -Z%d %s'", BraceLevel - 1, g_tex_name);
 
@@ -720,36 +787,54 @@ globals: g_tex_name;
     }
     *f = NULL;
     diagnostics(4, "Closed RTF file");
+    fprintf(stderr,"\n");
 }
 
-void putRtfChar(char cThis)
+void putRtfCharEscaped(char cThis)
 
 /****************************************************************************
-purpose: output filter to escape characters written to an RTF file
-		 all output to the RTF file passes through this routine or the one below
+purpose: output a single escaped character to the RTF file
+         this is primarily useful for the verbatim-like enviroments
  ****************************************************************************/
 {
-    if (cThis == '\\')
-        fprintf(fRtf, "\\\\");
+	if (getTexMode() == MODE_VERTICAL)
+		changeTexMode(MODE_HORIZONTAL);
+	if (cThis == '\\')
+        fprintRTF("%s","\\\\");
     else if (cThis == '{')
-        fprintf(fRtf, "\\{");
+        fprintRTF("%s", "\\{");
     else if (cThis == '}')
-        fprintf(fRtf, "\\}");
+        fprintRTF("%s", "\\}");
     else if (cThis == '\n')
-        fprintf(fRtf, "\n\\par ");
+        fprintRTF("%s", "\n\\par ");
     else
-        fputc(cThis, fRtf);
+        fprintRTF("%c",cThis);
 }
 
+/****************************************************************************
+purpose: output a string with escaped characters to the RTF file
+         this is primarily useful for the verbatim-like enviroments
+ ****************************************************************************/
+void putRtfStrEscaped(const char * string)
+{
+	char *s = (char *) string;
+	if (string == NULL) return;
+    while (*s) putRtfCharEscaped(*s++);
+}
+
+
 void fprintRTF(char *format, ...)
 
 /****************************************************************************
-purpose: output filter to track of brace depth and font settings
-		 all output to the RTF file passes through this routine or the one above
+purpose: output a formatted string to the RTF file.  It is assumed that the
+         formatted string has been properly escaped for the RTF file.  
+         *ALL* output to the RTF file passes through this routine.
  ****************************************************************************/
 {
     char buffer[1024];
     char *text;
+    char last='\0';
+    
     va_list apf;
 
     va_start(apf, format);
@@ -759,23 +844,19 @@ purpose: output filter to track of brace depth and font settings
 
     while (*text) {
 
-        if ((unsigned char) *text > 127) {
-
-            WriteEightBitChar(text[0]);
-
-        } else {
-            fputc(*text, fRtf);
-
-            if (*text == '{')
-                PushFontSettings();
-
-            if (*text == '}')
-                PopFontSettings();
-
-            if (*text == '\\')
-                MonitorFontChanges(text);
-        }
-        text++;
+		WriteEightBitChar(text[0], fRtf);
+	
+		if (*text == '{' && last != '\\')
+			PushFontSettings();
+	
+		if (*text == '}' && last != '\\')
+			PopFontSettings();
+	
+		if (*text == '\\' && last != '\\')
+			MonitorFontChanges(text);
+	
+		last= *text;
+		text++;
     }
 }
 
@@ -795,8 +876,8 @@ purpose: return the directory to store temporary files
     char pathsep_str[2] = { PATHSEP, 0 };   /* for os2 or w32 "unix" compiler */
 
     /* first use any temporary directory specified as an option */
-    if (g_tmp_path)
-        t = strdup(g_tmp_path);
+    if (g_tmp_dir)
+        t = strdup(g_tmp_dir);
 
     /* next try the environment variable TMPDIR */
     else if ((u = getenv("TMPDIR")) != NULL)
@@ -833,7 +914,7 @@ purpose: duplicate string --- exists to ease porting
         diagnostics(ERROR, "Cannot allocate memory to duplicate string");
     strcpy(s, str);
 
-/*	diagnostics(3,"ptr %x",(unsigned long)s);*/
+/*	diagnostics(5,"ptr %x",(unsigned long)s);*/
     return s;
 }
 
@@ -846,8 +927,6 @@ purpose: opens "g_home_dir/path"  and
     char *name;
     FILE *p;
 
-    diagnostics(3, "Opening <%s>, mode=[%s]", path, mode);
-
     if (path == NULL || mode == NULL)
         return (NULL);
 
@@ -856,14 +935,16 @@ purpose: opens "g_home_dir/path"  and
     else
         name = strdup_together(g_home_dir, path);
 
-    diagnostics(3, "Opening <%s>", name);
     p = fopen(name, mode);
 
-    if (p == NULL && strstr(path, ".tex"))
-        p = (FILE *) open_cfg(path, FALSE);
-
     if (p == NULL) {
-        diagnostics(WARNING, "Cannot open <%s>", name);
+    	if (strstr(path, ".tex") != NULL)
+        	p = (FILE *) open_cfg(path, FALSE);
+	} else
+    	diagnostics(2, "Opened '%s'", name);
+	
+    if (p == NULL) {
+        diagnostics(WARNING, "Cannot open '%s'", name);
         fflush(NULL);
     }
 
@@ -875,7 +956,7 @@ void debug_malloc(void)
 {
     char c;
 
-    diagnostics(1, "Malloc Debugging --- press return to continue");
+    diagnostics(WARNING, "Malloc Debugging --- press return to continue");
     fflush(NULL);
     fscanf(stdin, "%c", &c);
 }
