@@ -21,7 +21,7 @@ This file is available from http://sourceforge.net/projects/latex2rtf/
  
 Authors:
     1998-2000 Georg Lehner
-    2001-2002 Scott Prahl
+    2001-2007 Scott Prahl
 */
 
 #include <stdio.h>
@@ -33,9 +33,9 @@ Authors:
 #include "commands.h"
 #include "cfg.h"
 #include "stack.h"
-#include "util.h"
+#include "utils.h"
 #include "parser.h"
-#include "l2r_fonts.h"
+#include "fonts.h"
 #include "lengths.h"
 #include "definitions.h"
 #include "funct1.h"
@@ -94,11 +94,10 @@ void PopTrackLineNumber(void)
     g_track_line_number--;
 }
 
-int CurrentLineNumber(void)
-
 /***************************************************************************
  purpose:     returns the current line number of the text being processed
 ****************************************************************************/
+int CurrentLineNumber(void)
 {
     return g_parser_line;
 }
@@ -119,6 +118,18 @@ void UpdateLineNumber(char *s)
     }
 }
 
+/***************************************************************************
+ purpose:     returns the current file descriptor
+****************************************************************************/
+int CurrentFileDescriptor(void)
+{
+    int fd=0;
+    if (g_parser_file)
+    	fd = fileno(g_parser_file);
+    
+    return fd;
+}
+
 char *CurrentFileName(void)
 
 /***************************************************************************
@@ -137,7 +148,7 @@ char *CurrentFileName(void)
 	The following two routines allow parsing of multiple files and strings
 */
 
-int PushSource(char *filename, char *string)
+int PushSource(const char *filename, const char *string)
 
 /***************************************************************************
  purpose:     change the source used by getRawTexChar() to either file or string
@@ -152,15 +163,15 @@ int PushSource(char *filename, char *string)
     int line = 1;
 
     if (0) {
-        diagnostics(1, "Before PushSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
+        diagnostics(WARNING, "Before PushSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
           g_parser_line, g_parser_depth, g_parser_include_level);
         for (i = 0; i <= g_parser_depth; i++) {
             if (g_parser_stack[i].file)
-                diagnostics(1, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
+                diagnostics(WARNING, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
 
             else {
-                strncpy(s, g_parser_stack[i].string, 25);
-                diagnostics(1, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
+                strncpy_printable(s, g_parser_stack[i].string, 25);
+                diagnostics(WARNING, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
             }
         }
     }
@@ -180,7 +191,7 @@ int PushSource(char *filename, char *string)
 
         /* if not then try to open a file */
     } else if (filename) {
-        p = my_fopen(filename, "rb");
+        p = my_fopen((char *)filename, "rb");
         if (p == NULL)
             return 1;
         g_parser_include_level++;
@@ -207,22 +218,22 @@ int PushSource(char *filename, char *string)
     g_parser_string = g_parser_stack[g_parser_depth].string;
 
     if (g_parser_file) {
-        diagnostics(5, "Opening Source File %s", g_parser_stack[g_parser_depth].file_name);
+        diagnostics(3, "Opening Source File %s", g_parser_stack[g_parser_depth].file_name);
     } else {
-        strncpy(s, g_parser_string, 25);
-        diagnostics(5, "Opening Source string <%s>", s);
+        diagnostics(4, "Opening Source string");
+        show_string(5, g_parser_string, "opening");
     }
 
     if (0) {
-        diagnostics(1, "After PushSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
+        diagnostics(WARNING, "After PushSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
           g_parser_line, g_parser_depth, g_parser_include_level);
         for (i = 0; i <= g_parser_depth; i++) {
             if (g_parser_stack[i].file)
-                diagnostics(1, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
+                diagnostics(WARNING, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
 
             else {
-                strncpy(s, g_parser_stack[i].string, 25);
-                diagnostics(1, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
+                strncpy_printable(s, g_parser_stack[i].string, 25);
+                diagnostics(WARNING, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
             }
         }
     }
@@ -264,21 +275,21 @@ void PopSource(void)
         diagnostics(ERROR, "More PopSource() calls than PushSource() ");
 
     if (0) {
-        diagnostics(1, "Before PopSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
+        diagnostics(WARNING, "Before PopSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
           g_parser_line, g_parser_depth, g_parser_include_level);
         for (i = 0; i <= g_parser_depth; i++) {
             if (g_parser_stack[i].file)
-                diagnostics(1, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
+                diagnostics(WARNING, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
 
             else {
-                strncpy(s, g_parser_stack[i].string, 25);
-                diagnostics(1, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
+                strncpy_printable(s, g_parser_stack[i].string, 25);
+                diagnostics(WARNING, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
             }
         }
     }
 
     if (g_parser_file) {
-        diagnostics(5, "Closing Source File %s", g_parser_stack[g_parser_depth].file_name);
+        diagnostics(3, "Closing Source File '%s'", g_parser_stack[g_parser_depth].file_name);
         fclose(g_parser_file);
         free(g_parser_stack[g_parser_depth].file_name);
         g_parser_stack[g_parser_depth].file_name = NULL;
@@ -293,7 +304,7 @@ void PopSource(void)
             s[49] = '\0';
         }
 
-        diagnostics(5, "Closing Source string <%s>", s);
+        show_string(5, s, "closing");
         free(g_parser_stack[g_parser_depth].string_start);
         g_parser_stack[g_parser_depth].string_start = NULL;
     }
@@ -310,31 +321,107 @@ void PopSource(void)
     }
 
     if (g_parser_file)
-        diagnostics(5, "Resuming Source File %s", g_parser_stack[g_parser_depth].file_name);
+        diagnostics(4, "Resuming Source File '%s'", g_parser_stack[g_parser_depth].file_name);
     else {
-        strncpy(s, g_parser_string, 25);
-        diagnostics(5, "Resuming Source string <%s>", s);
+        diagnostics(5, "Resuming Source string");
+        show_string(5,g_parser_string,"resuming");
     }
 
     if (0) {
-        diagnostics(1, "After PopSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
+        diagnostics(WARNING, "After PopSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
           g_parser_line, g_parser_depth, g_parser_include_level);
         for (i = 0; i <= g_parser_depth; i++) {
             if (g_parser_stack[i].file)
-                diagnostics(1, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
+                diagnostics(WARNING, "i=%d file   =%s, line=%d", i, g_parser_stack[i].file_name, g_parser_stack[i].file_line);
 
             else {
-                strncpy(s, g_parser_stack[i].string, 25);
-                diagnostics(1, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
+                strncpy_printable(s, g_parser_stack[i].string, 25);
+                diagnostics(WARNING, "i=%d string =%s, line=%d", i, s, g_parser_stack[i].file_line);
             }
         }
     }
 }
 
+void CmdInclude(int code)
+
+/******************************************************************************
+ purpose: handles \input file, \input{file}, \include{file}
+          code == 0 for \include
+          code == 1 for \input
+ ******************************************************************************/
+{
+    int cNext;
+    char name[100];
+    int i;
+    char *basename=NULL;
+    char *texname=NULL;
+
+    cNext = getNonSpace();
+
+    if (cNext == '{') {         /* \input{gnu} or \include{gnu} */
+        ungetTexChar(cNext);
+        basename = getBraceParam();
+
+    } else {                    /* \input gnu */
+        i = 0;
+        while (cNext != '\0' && !isspace(cNext)) {
+        	if (i<99) name[i] = (char) cNext;
+        	i++;
+        	cNext = getTexChar();
+        }
+        
+        if (i<99) 
+        	name[i] = '\0';
+		else {
+        	name[99] = '\0';
+        	diagnostics(WARNING, "\\input filename '%s' more than 100 chars, skipping",name);
+        	return;
+        }
+ 
+        basename = strdup(name);
+    }
+
+	if (strstr(basename, "german.sty") != NULL) {
+    	GermanMode = TRUE;
+     	PushEnvironment(GERMAN_MODE);
+     	free(basename);
+     	return;
+
+    } else if (strstr(basename, "french.sty") != NULL) {
+        FrenchMode = TRUE;
+        PushEnvironment(FRENCH_MODE);
+     	free(basename);
+     	return;
+	}
+	
+    if (basename && strstr(basename, ".tex") == NULL)         /* append .tex if missing */
+        texname = strdup_together(basename, ".tex");
+
+    if (texname && PushSource(texname, NULL) == 0)            /* Try the .tex name first*/
+        diagnostics(WARNING, "Including file <%s> (.tex appended)", texname);
+      
+    else if (basename && PushSource(basename, NULL) == 0)     /* Try the basename second*/
+        diagnostics(WARNING, "Including file <%s>", basename);
+
+	/* \include{file} always starts a new page */
+	if (code == 0)
+		PushSource(NULL, "\\pagebreak ");
+		
+    if (basename) free(basename);
+    if (texname)  free(texname);
+}
+
+
 #define CR (char) 0x0d
 #define LF (char) 0x0a
 
-char getRawTexChar()
+int getParserDepth(void)
+{
+	return g_parser_depth;
+}
+
+
+char getRawTexChar(void)
 
 /***************************************************************************
  purpose:     get the next character from the input stream with minimal
@@ -347,7 +434,7 @@ char getRawTexChar()
 
     if (g_parser_file) {
         thechar = getc(g_parser_file);
-        if (thechar == EOF)
+        while (thechar == EOF) {
             if (!feof(g_parser_file))
                 diagnostics(ERROR, "Unknown file I/O error reading latex file\n");
             else if (g_parser_include_level > 1) {
@@ -355,7 +442,8 @@ char getRawTexChar()
                 thechar = getRawTexChar();  /* get next char from parent file */
             } else
                 thechar = '\0';
-        else if (thechar == CR) {   /* convert CR, CRLF, or LF to \n */
+        }
+        if (thechar == CR) {   /* convert CR, CRLF, or LF to \n */
             thechar = getc(g_parser_file);
             if (thechar != LF && !feof(g_parser_file))
                 ungetc(thechar, g_parser_file);
@@ -367,10 +455,30 @@ char getRawTexChar()
 
         g_parser_currentChar = (char) thechar;
 
-    } else {                    /* no need to sanitize strings! */
+    } else {
+
         if (g_parser_string && *g_parser_string) {
-            g_parser_currentChar = *g_parser_string;
+			thechar = *g_parser_string;
+
+			/* convert CR, CRLF, or LF to \n */			
+			if (thechar == CR) {   
+				g_parser_string++;
+				thechar = *g_parser_string;
+				if (thechar != LF)
+					g_parser_string--;
+				thechar = '\n';
+			} else if (thechar == LF)
+				thechar = '\n';
+			else if (thechar == '\t')
+            	thechar = ' ';
+            	
+            g_parser_currentChar = thechar;
             g_parser_string++;
+        } 
+        else if (g_parser_depth > 15) 
+        {
+             PopSource();    /* go back to parsing parent */
+             g_parser_currentChar = getRawTexChar();  /* get next char from parent file */
         } else
             g_parser_currentChar = '\0';
     }
@@ -380,6 +488,15 @@ char getRawTexChar()
 
     g_parser_penultimateChar = g_parser_lastChar;
     g_parser_lastChar = g_parser_currentChar;
+    if (0) {
+		if (g_parser_currentChar=='\n')
+			diagnostics(5,"getRawTexChar = <\\n>");
+		else if (g_parser_currentChar=='\0')
+			diagnostics(5,"getRawTexChar = <\\0> depth=%d, files=%d", g_parser_depth, g_parser_include_level);
+		else
+			diagnostics(5,"getRawTexChar = <%2c>",g_parser_currentChar);
+	}
+	/* if (g_parser_currentChar=='\0') exit(0);*/
     return g_parser_currentChar;
 }
 
@@ -440,7 +557,14 @@ char getTexChar()
         g_parser_backslashes++;
     else
         g_parser_backslashes = 0;
-    diagnostics(6, "after getTexChar=<%c> backslashes=%d line=%d", cThis, g_parser_backslashes, g_parser_line);
+	if (0) {
+		if (cThis=='\n')
+			diagnostics(6,"getRawTexChar = <\\n> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
+		else if (cThis=='\0')
+			diagnostics(6,"getRawTexChar = <\\0> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
+		else
+			diagnostics(6,"getRawTexChar = <%2c> backslashes=%d line=%ld",cThis, g_parser_backslashes, g_parser_line);
+	}
     return cThis;
 }
 
@@ -465,7 +589,9 @@ char getNonBlank(void)
 {
     char c;
 
-    while ((c = getTexChar()) && (c == ' ' || c == '\n')) {
+	c = getTexChar();
+    while (c == ' ' || c == '\n') {
+    	c = getTexChar();
     }
     return c;
 }
@@ -484,7 +610,6 @@ char getNonSpace(void)
 }
 
 void skipSpaces(void)
-
 /***************************************************************************
  Description: skip to the next non-space character from the input stream
 ****************************************************************************/
@@ -496,6 +621,16 @@ void skipSpaces(void)
     ungetTexChar(c);
 }
 
+void	skipWhiteSpace(void)
+/***************************************************************************
+ Description: skip over spaces and linefeeds
+****************************************************************************/
+{
+    char c=getNonBlank();
+    ungetTexChar(c);
+}
+
+
 int getSameChar(char c)
 
 /***************************************************************************
@@ -598,7 +733,7 @@ void CmdIgnoreParameter(int code)
     int regParmCount = code % 10;
     char cThis;
 
-    diagnostics(4, "CmdIgnoreParameter [%d] {%d}", optParmCount, regParmCount);
+    diagnostics(5, "CmdIgnoreParameter [%d] {%d}", optParmCount, regParmCount);
 
     while (regParmCount) {
         cThis = getNonBlank();
@@ -641,7 +776,8 @@ char *getSimpleCommand(void)
 
 /**************************************************************************
      purpose: returns a simple command e.g., \alpha\beta will return "\beta"
-                                                   ^
+                                                  ^
+                                              \! will return \!
  **************************************************************************/
 {
     char buffer[128];
@@ -652,7 +788,9 @@ char *getSimpleCommand(void)
     if (buffer[0] != '\\')
         return NULL;
 
-    for (size = 1; size < 127; size++) {
+	buffer[1] = getTexChar();
+
+    for (size = 2; size < 127; size++) {
         buffer[size] = getRawTexChar(); /* \t \r '%' all end command */
 
         if (!isalpha((int) buffer[size])) {
@@ -689,19 +827,19 @@ char *getBracketParam(void)
 
     if (c == '[') {
         text = getDelimitedText('[', ']', FALSE);
-        diagnostics(5, "getBracketParam [%s]", text);
+        diagnostics(6, "getBracketParam [%s]", text);
 
     } else {
         ungetTexChar(c);
         text = NULL;
-        diagnostics(5, "getBracketParam []");
+        diagnostics(6, "getBracketParam []");
     }
 
     PopTrackLineNumber();
     return text;
 }
 
-char *getBraceParam(void)
+static char *getBraceParam0(int raw_flag)
 
 /**************************************************************************
      purpose: allocates and returns the next parameter in the LaTeX file
@@ -730,7 +868,7 @@ char *getBraceParam(void)
         text = getSimpleCommand();
 
     } else if (s[0] == '{')
-        text = getDelimitedText('{', '}', TRUE);
+        text = getDelimitedText('{', '}', raw_flag);
 
     else {
         s[1] = '\0';
@@ -738,10 +876,20 @@ char *getBraceParam(void)
     }
 
     PopTrackLineNumber();
-    diagnostics(5, "Leaving getBraceParam {%s}", text);
+    diagnostics(6, "Leaving getBraceParam {%s}", text);
     return text;
 }
 
+char *getBraceParam(void)
+{
+	return getBraceParam0(FALSE);
+}
+
+char *getBraceRawParam(void)
+{
+	return getBraceParam0(TRUE);
+}
+
 char *getLeftRightParam(void)
 
 /**************************************************************************
@@ -841,13 +989,110 @@ char *getTexUntil(char *target, int raw)
 
     if (!end_of_file_reached)   /* do not include target in returned string */
         buffer[i - len] = '\0';
+    else {
+        diagnostics(ERROR, "Could not find <%s>", target);
+        exit(1);
+    }
 
     PopTrackLineNumber();
 
-    diagnostics(3, "buffer size =[%d], actual=[%d]", strlen(buffer), i - len);
+    diagnostics(6, "buffer size =[%d], actual=[%d]", strlen(buffer), i - len);
 
     s = strdup(buffer);
-    diagnostics(3, "strdup result = %s", s);
+    diagnostics(6, "getTexUntil result = %s", s);
+    return s;
+}
+
+char *getSpacedTexUntil(char *target, int raw)
+
+/**************************************************************************
+     purpose: returns the portion of the file to the beginning of target
+
+     getSpacedTexUntil("\begin|{|document|}")
+     
+     will match the regular expression "\\begin *{ *document *}"
+ **************************************************************************/
+{
+    enum { BUFFSIZE = 16000 };
+    char buffer[BUFFSIZE];
+    char *s;
+    int buffer_pos, target_pos, target_len, max_buffer_pos, start_pos;
+    bool end_of_file_reached = FALSE;
+	bool matched;
+	
+    PushTrackLineNumber(FALSE);
+
+    diagnostics(5, "getSpaceTexUntil target = <%s> raw_search = %d ", target, raw);
+
+	matched = FALSE;	
+	buffer_pos = 0;
+	target_pos = 0;
+	start_pos  = 0;
+	target_len = strlen(target);
+	max_buffer_pos = -1;
+	
+    do {
+        
+        /* the next character might already be in the buffer */
+        if (buffer_pos > max_buffer_pos) {
+            buffer[buffer_pos] = (raw) ? getRawTexChar() : getTexChar();
+            max_buffer_pos = buffer_pos;
+        } 
+
+        if (buffer[buffer_pos] == '\0') {
+            end_of_file_reached = TRUE;
+            diagnostics(ERROR, "end of file reached before '%s' was found",target);
+        }
+
+        if (buffer[buffer_pos] == target[target_pos]) {
+        	if (target_pos == 0) 
+        		start_pos = buffer_pos;
+        	target_pos++;
+        } 
+        
+        /* does not match next character in target ... */
+        else if (target[target_pos] != '|') {
+
+            if (target_pos > 0)        /* false start, put back what was found */
+                buffer_pos = start_pos;
+            target_pos = 0;
+        
+        /* next character in target is '|' */ 
+        } else if (buffer[buffer_pos] != ' ' && buffer[buffer_pos] != '\n') {
+        	
+			/* next char is non-blank ... either match or reset */
+			target_pos++;  /* move past wildcard */
+			if (buffer[buffer_pos] == target[target_pos]) {
+				target_pos++;
+			} else {
+				buffer_pos = start_pos;
+				target_pos = 0;
+			}
+        }
+        
+        if (0) {
+		if (buffer[buffer_pos] != '\n')
+			diagnostics(WARNING, "this char = <%c>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
+		else
+			diagnostics(WARNING, "this char = <\\n>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
+		}
+		
+        buffer_pos++;
+
+
+		if (buffer_pos == BUFFSIZE)
+			diagnostics(ERROR, "Could not find <%s> in %d characters \n\
+			Recompile with larger BUFFSIZE in getTexUntil() in parser.c", target, BUFFSIZE);
+
+    } while (target_pos < target_len);
+
+	/* terminate buffer */
+    buffer[start_pos] = '\0';
+
+    PopTrackLineNumber();
+
+    s = strdup(buffer);
+    diagnostics(6, "getSpacedTexUntil result = %s", s);
     return s;
 }
 
@@ -906,7 +1151,7 @@ int getDimension(void)
 
     if (i == 19 || sscanf(buffer, "%f", &num) != 1) {
         diagnostics(WARNING, "Screwy number in TeX dimension");
-        diagnostics(1, "getDimension() number is <%s>", buffer);
+        diagnostics(WARNING, "getDimension() number is <%s>", buffer);
         return 0;
     }
 
@@ -916,6 +1161,9 @@ int getDimension(void)
     skipSpaces();
     buffer[0] = tolower((int) getTexChar());
 
+	if (buffer[0] == '\0')  /* no units specified ... assume points */
+        return (int) (num * 20);
+	
 /* skip "true" */
     if (buffer[0] == 't') {
         cThis = getTexChar();
@@ -975,460 +1223,3 @@ int getDimension(void)
 
 }
 
-#define SECTION_BUFFER_SIZE 2048
-static char *section_buffer = NULL;
-static size_t section_buffer_size = SECTION_BUFFER_SIZE;
-
-static void increase_buffer_size(void)
-{
-    char *new_section_buffer;
-
-    new_section_buffer = malloc(2 * section_buffer_size + 1);
-    if (new_section_buffer == NULL)
-        diagnostics(ERROR, "Could not allocate enough memory to process file. Sorry.");
-    memmove(new_section_buffer, section_buffer, section_buffer_size);
-    section_buffer_size *= 2;
-    free(section_buffer);
-    section_buffer = new_section_buffer;
-    diagnostics(4, "Expanded buffer size is now %ld", section_buffer_size);
-}
-
-void getSection(char **body, char **header, char **label)
-
-/**************************************************************************
-	purpose: obtain the next section of the latex file
-	
-	This is now a preparsing routine that breaks a file up into sections.  
-	Macro expansion happens here as well.  \input and \include is also
-	handled here.  The reason for this routine is allow \labels to refer
-	to sections.  
-	
-	This routine reads text until a new section heading is found.  The text 
-	is returned in body and the *next* header is returned in header.  If 
-	no header follows then NULL is returned.
-	
-**************************************************************************/
-{
-    int possible_match, found;
-    char cNext, *s, *text, *next_header, *str;
-    int i;
-    size_t delta;
-    int match[35];
-    char *command[35] = { "",   /* 0 entry is for user definitions */
-        "",                     /* 1 entry is for user environments */
-        "\\begin{verbatim}", "\\begin{figure}", "\\begin{figure*}", "\\begin{equation}",
-        "\\begin{eqnarray}", "\\begin{table}", "\\begin{description}", "\\begin{comment}",
-        "\\end{verbatim}", "\\end{figure}", "\\end{figure*}", "\\end{equation}",
-        "\\end{eqnarray}", "\\end{table}", "\\end{description}", "\\end{comment}",
-        "\\part", "\\chapter", "\\section", "\\subsection", "\\subsubsection",
-        "\\section*", "\\subsection*", "\\subsubsection*",
-        "\\label", "\\input", "\\include", "\\verb", "\\url",
-        "\\newcommand", "\\def", "\\renewcommand", "\\endinput",
-    };
-
-    int ncommands = 35;
-
-    const int b_verbatim_item = 2;
-    const int b_figure_item = 3;
-    const int b_figure_item2 = 4;
-    const int b_equation_item = 5;
-    const int b_eqnarray_item = 6;
-    const int b_table_item = 7;
-    const int b_description_item = 8;
-    const int b_comment_item = 9;
-    const int e_verbatim_item = 10;
-    const int e_figure_item = 11;
-    const int e_equation_item = 12;
-    const int e_equation_item2 = 13;
-    const int e_eqnarray_item = 14;
-    const int e_table_item = 15;
-    const int e_description_item = 16;
-    const int e_comment_item = 17;
-
-    const int label_item = 26;
-    const int input_item = 27;
-    const int include_item = 28;
-    const int verb_item = 29;
-    const int url_item = 30;
-    const int new_item = 31;
-    const int def_item = 32;
-    const int renew_item = 33;
-    const int endinput_item = 34;
-
-    int bs_count = 0;
-    size_t index = 0;
-    int label_depth = 0;
-    int n_target = strlen(InterpretCommentString);
-
-    if (section_buffer == NULL) {
-        section_buffer = malloc(section_buffer_size + 1);
-        if (section_buffer == NULL)
-            diagnostics(ERROR, "Could not allocate enough memory to process file. Sorry.");
-    }
-
-    text = NULL;
-    next_header = NULL;         /* typically becomes \subsection{Cows eat grass} */
-    *body = NULL;
-    *header = NULL;
-    *label = NULL;
-
-    PushTrackLineNumber(FALSE);
-    for (delta = 0;; delta++) {
-
-        if (delta + 2 >= section_buffer_size)
-            increase_buffer_size();
-
-        cNext = getRawTexChar();
-        while (cNext == '\0' && g_parser_depth > 0) {
-            PopSource();
-            cNext = getRawTexChar();
-        }
-
-        if (cNext == '\0')
-            diagnostics(5, "[%ld] xchar=000 '\\0' (backslash count=%d)", delta, bs_count);
-        else if (cNext == '\n')
-            diagnostics(5, "[%ld] xchar=012 '\\n' (backslash count=%d)", delta, bs_count);
-        else
-            diagnostics(5, "[%ld] xchar=%03d '%c' (backslash count=%d)", delta, (int) cNext, cNext, bs_count);
-
-        /* add character to buffer */
-        *(section_buffer + delta) = cNext;
-
-        if (cNext == '\0')
-            break;
-
-        /* slurp TeX comments but discard InterpretCommentString */
-        if (cNext == '%' && even(bs_count)) {
-            int n = 0;
-
-            delta++;
-            *(section_buffer + delta) = cNext;
-            cNext = getRawTexChar();
-
-            while (cNext != '\n' && cNext != '\0') {
-
-                delta++;
-                n++;
-                *(section_buffer + delta) = cNext;
-
-                if (delta + 2 >= section_buffer_size)
-                    increase_buffer_size();
-
-                /* handle %latex2rtf: */
-                if (n == n_target && strncmp(InterpretCommentString, section_buffer + delta - n + 1, n_target) == 0)
-                    break;
-
-                cNext = getRawTexChar();
-            }
-
-            delta -= n + 2;     /* remove '% .... \n', or '% ... \0' or just '%latex2rtf:' */
-            continue;           /* go get the next character */
-        }
-
-        /* begin search if backslash found */
-        if (*(section_buffer + delta) == '\\') {
-            bs_count++;
-            if (odd(bs_count)) {    /* avoid "\\section" and "\\\\section" */
-                for (i = 0; i < ncommands; i++)
-                    match[i] = TRUE;
-                index = 1;
-                continue;
-            }
-        } else
-            bs_count = 0;
-
-        if (index == 0)
-            continue;
-
-        possible_match = FALSE;
-
-/* slow... */
-        if (match[0])           /* do any user defined commands possibly match? */
-            match[0] = maybeDefinition(section_buffer + delta - index + 1, index - 1);
-
-        /* do any user defined commands possibly match? */
-        if (match[1])
-            match[1] = maybeEnvironment(section_buffer + delta - index, index - 1);
-
-        possible_match = match[0] || match[1];
-
-        for (i = 2; i < ncommands; i++) {   /* test each command for match */
-            if (!match[i])
-                continue;
-
-            if (*(section_buffer + delta) != command[i][index]) {
-                match[i] = FALSE;
-
-/*				diagnostics(2,"index = %d, char = %c, failed to match %s, size=%d", \
-				index,*p,command[i],strlen(command[i]));
-*/ continue;
-            }
-            possible_match = TRUE;
-        }
-
-        found = FALSE;
-
-        if (match[0]) {         /* expand user macros */
-            cNext = getRawTexChar();    /* wrong when cNext == '%' */
-            ungetTexChar(cNext);
-            if (!isalpha((int) cNext) && index > 1) {   /* is macro name complete? */
-
-                *(section_buffer + delta + 1) = '\0';
-                i = existsDefinition(section_buffer + delta - index + 1);
-                if (i > -1) {
-                    if (cNext == ' ') {
-                        cNext = getNonSpace();
-                        ungetTexChar(cNext);
-                    }
-
-                    delta -= index + 1; /* remove \macroname */
-                    str = expandDefinition(i);
-                    diagnostics(4, "getSection() expanded macro string is <%s>", str);
-                    PushSource(NULL, str);
-                    free(str);
-                    index = 0;
-                    continue;
-                }
-            }
-        }
-
-        if (match[1]) {         /* expand user environments */
-            char *p = section_buffer + delta - index;
-
-            cNext = getRawTexChar();    /* wrong when cNext == '%' */
-            str = NULL;
-
-            if (cNext == '}' && index > 5) {    /* is environ name complete? */
-                *(p + index + 1) = '\0';
-                if (*(p + 1) == 'e') {  /* find \\end{userenvironment} */
-                    i = existsEnvironment(p + strlen("\\end{"));
-                    str = expandEnvironment(i, CMD_END);
-                } else if (index > 8) { /* find \\begin{userenvironment} */
-                    i = existsEnvironment(p + strlen("\\begin{"));
-                    str = expandEnvironment(i, CMD_BEGIN);
-                }
-            }
-
-            if (str) {          /* found */
-                char *str2;
-
-                diagnostics(4, "matched <%s}>", p);
-                diagnostics(4, "expanded <%s>", str);
-                if (*(p + 1) == 'e')
-                    str2 = strdup_together(str, "}");
-                else
-                    str2 = strdup_together("{", str);
-                free(str);
-                PushSource(NULL, str2);
-                free(str2);
-                delta -= index + 1; /* remove \begin{userenvironment} */
-                index = 0;
-                diagnostics(4, "getSection() expanded environment string is <%s>", str);
-                continue;
-            }
-
-            ungetTexChar(cNext);    /* put the character back */
-        }
-
-        for (i = 2; i < ncommands; i++) {   /* discover any exact matches */
-            if (!match[i])
-                continue;
-            if (index + 1 == strlen(command[i])) {
-                found = TRUE;
-                break;
-            }
-        }
-
-        if (found) {            /* make sure the next char is the right sort */
-            diagnostics(5, "matched %s", command[i]);
-            cNext = getRawTexChar();
-            ungetTexChar(cNext);
-
-            if (i > e_description_item && i <= include_item && cNext != ' ' && cNext != '{') {
-                found = FALSE;
-                match[i] = FALSE;
-                diagnostics(5, "oops! did not match %s", command[i]);
-            }
-        }
-
-        if (!possible_match) {  /* no possible matches, reset and wait for next '\\' */
-            index = 0;
-            continue;
-        } else
-            index++;
-
-        if (!found)
-            continue;
-
-        if (i == endinput_item) {
-            delta -= 9;         /* remove \endinput */
-            PopSource();
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        if (i == verb_item || i == url_item) {  /* slurp \verb#text# */
-            if (i == url_item && cNext == '{')
-                cNext = '}';
-            delta++;
-            *(section_buffer + delta) = getRawTexChar();
-            delta++;
-            while ((*(section_buffer + delta) = getRawTexChar()) != '\0' && *(section_buffer + delta) != cNext) {
-                delta++;
-                if (delta >= section_buffer_size)
-                    increase_buffer_size();
-            }
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        if (i == input_item || i == include_item) {
-            char *s, *s2;
-
-            s = getBraceParam();
-            if (i == input_item)
-                diagnostics(4, "\\input{%s}", s);
-            else
-                diagnostics(4, "\\include{%s}", s);
-
-            if (strstr(s, "german.sty") != NULL) {
-                GermanMode = TRUE;
-                PushEnvironment(GERMAN_MODE);
-
-            } else if (strstr(s, "french.sty") != NULL) {
-                FrenchMode = TRUE;
-                PushEnvironment(FRENCH_MODE);
-
-            } else if (strcmp(s, "") == 0) {
-                diagnostics(WARNING, "Empty or invalid filename in \\include{}");
-
-            } else {
-
-                if (strstr(s, ".ltx") == NULL && strstr(s, ".tex") == NULL) {
-                    /* extension .tex is appended automatically if missing */
-                    s2 = strdup_together(s, ".tex");
-                    free(s);
-                    s = s2;
-                }
-
-                PushSource(s, NULL);    /* ignore return value */
-            }
-            delta -= (i == input_item) ? 6 : 8; /* remove \input or \include */
-            free(s);
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        if (i == label_item) {
-            s = getBraceParam();
-            diagnostics(4, "\\label{%s}", s);
-
-            /* append \label{tag} to the buffer */
-            delta++;
-            *(section_buffer + delta) = '{';
-            while (delta + strlen(s) + 1 >= section_buffer_size)
-                increase_buffer_size();
-            strcpy(section_buffer + delta + 1, s);
-            delta += strlen(s) + 1;
-            *(section_buffer + delta) = '}';
-
-            if (!(*label) && strlen(s) && label_depth == 0)
-                *label = strdup_nobadchars(s);
-
-            free(s);
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        /* process any new definitions */
-        if (i == def_item || i == new_item || i == renew_item) {
-            cNext = getRawTexChar();    /* wrong when cNext == '%' */
-            ungetTexChar(cNext);
-            if (isalpha((int) cNext))   /* is macro name complete? */
-                continue;
-
-            delta -= strlen(command[i]);    /* do not include in buffer */
-
-            if (i == def_item)
-                CmdNewDef(DEF_DEF);
-            else if (i == new_item)
-                CmdNewDef(DEF_NEW);
-            else
-                CmdNewDef(DEF_RENEW);
-
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        if (i == b_figure_item || i == b_figure_item2 || i == b_equation_item || i == b_eqnarray_item ||
-          i == b_table_item || i == b_description_item) {
-            label_depth++;      /* labels now will not be the section label */
-            index = 0;
-            continue;
-        }
-
-        if (i == e_figure_item || i == e_equation_item || i == e_equation_item2 || i == e_eqnarray_item ||
-          i == e_table_item || i == e_description_item) {
-            label_depth--;      /* labels may now be the section label */
-            index = 0;
-            continue;
-        }
-
-        if (i == b_verbatim_item) { /* slurp environment to avoid inside */
-            delta++;
-            s = getTexUntil(command[e_verbatim_item], TRUE);
-
-            while (delta + strlen(s) + strlen(command[e_verbatim_item]) + 1 >= section_buffer_size)
-                increase_buffer_size();
-
-            strcpy(section_buffer + delta, s);  /* append s */
-            delta += strlen(s);
-
-            strcpy(section_buffer + delta, command[e_verbatim_item]);   /* append command[i] */
-            delta += strlen(command[e_verbatim_item]) - 1;
-            free(s);
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        if (i == b_comment_item) {  /* slurp environment to avoid inside */
-            delta++;
-            s = getTexUntil(command[e_comment_item], TRUE);
-
-            while (delta + strlen(s) + strlen(command[e_comment_item]) + 1 >= section_buffer_size)
-                increase_buffer_size();
-
-            strcpy(section_buffer + delta, s);  /* append s */
-            delta += strlen(s);
-
-            strcpy(section_buffer + delta, command[e_comment_item]);    /* append command[i] */
-            delta += strlen(command[e_comment_item]) - 1;
-            free(s);
-            index = 0;          /* keep looking */
-            continue;
-        }
-
-        diagnostics(5, "possible end of section");
-        diagnostics(5, "label_depth = %d", label_depth);
-
-        if (label_depth > 0)    /* still in a \begin{xxx} environment? */
-            continue;
-
-        /* actually found command to end the section */
-        diagnostics(2, "getSection found command to end section");
-        s = getBraceParam();
-        next_header = malloc(strlen(command[i]) + strlen(s) + 3);
-        strcpy(next_header, command[i]);
-        strcpy(next_header + strlen(command[i]), "{");
-        strcpy(next_header + strlen(command[i]) + 1, s);
-        strcpy(next_header + strlen(command[i]) + 1 + strlen(s), "}");
-        free(s);
-        delta -= strlen(command[i]) - 1;
-        *(section_buffer + delta) = '\0';
-        break;
-    }
-    text = strdup(section_buffer);
-    *body = text;
-    *header = next_header;
-    PopTrackLineNumber();
-}
