@@ -25,7 +25,7 @@ This file is available from http://sourceforge.net/projects/latex2rtf/
 #include <stdlib.h>
 #include "main.h"
 #include "convert.h"
-#include "l2r_fonts.h"
+#include "fonts.h"
 #include "commands.h"
 #include "funct1.h"
 #include "tables.h"
@@ -33,22 +33,15 @@ This file is available from http://sourceforge.net/projects/latex2rtf/
 #include "cfg.h"
 #include "parser.h"
 #include "counters.h"
-#include "util.h"
+#include "utils.h"
 #include "lengths.h"
-
-typedef struct TabularT {
-    int n;                      /* number of columns */
-    char *align;                /* align[1] is the alignment of the first column */
-    int *vert;                  /* vert[0] is the number of |'s before first column vert[1] is the number of |'s after
-                                   first column vert[n] is the number of |'s after last column */
-    int *width;                 /* width[1] is the width of the first column */
-    int *cline;                 /* cline[1] is true when line should be draws across column 1 */
-} TabularT;
+#include "preamble.h"
+#include "graphics.h"
+#include "vertical.h"
 
 int g_tabbing_left_position = 0;
 int g_tabbing_current_position = 0;
 int *g_tabbing_locations = NULL;
-
 int tabcounter = 0;
 bool tabbing_return = FALSE;
 bool tabbing_on_itself = FALSE;
@@ -57,9 +50,16 @@ bool g_processing_tabbing = FALSE;
 bool g_processing_tabular = FALSE;
 bool g_processing_table = FALSE;
 
-int colCount;                   /* number of columns in a tabular environment */
-int actCol;                     /* actual column in the tabular environment */
-char *colFmt = NULL;
+typedef struct TabularT {
+    int n;                /* number of columns */
+    int i;                /* current column */
+    char *align;          /* align[1] is the alignment of the first column */
+    int *vert;            /* vert[0] is the number of |'s before first column 
+                             vert[1] is the number of |'s after first column
+                             vert[n] is the number of |'s after last column */
+    int *width;           /* width[1] is the width of the first column */
+    int *cline;           /* cline[1] is true when line should be draws across column 1 */
+} TabularT;
 
 void CmdTabjump(void)
 {
@@ -70,7 +70,7 @@ void CmdTabset(void)
 {
 }
 
-static void BeginCell(char align)
+static void BeginCellRtf(char align)
 
 /******************************************************************************
  purpose:  emit RTF code to start each cell
@@ -79,7 +79,7 @@ static void BeginCell(char align)
     fprintRTF("\\pard\\intbl\\q%c ", align);
 }
 
-static void EndCell(void)
+static void EndCellRtf(void)
 
 /******************************************************************************
  purpose:  emit RTF code to end each cell
@@ -94,59 +94,58 @@ static char *ConvertFormatString(char *s)
  purpose: convert latex formatting to something simpler
  ******************************************************************************/
 {
-    int braces, i;
-    char *simple;
+    int iCol, width;
+    char *simple, *t;
+    static int warned_once = FALSE;
 
     simple = strdup(s);         /* largest possible */
-    diagnostics(4, "Entering ConvertFormatString, input=<%s>", s);
+    diagnostics(5, "Entering ConvertFormatString, input=<%s>", s);
 
-    i = 0;
+    iCol = 0;
     while (*s) {
 
         switch (*s) {
             case 'c':
             case 'r':
             case 'l':
-                simple[i++] = *s;
+                simple[iCol] = *s;
+                iCol++;
                 break;
             case '{':          /* skip to balancing brace */
-                braces = 1;
-                s++;
-                while (*s && (braces > 1 || *s != '}')) {
-                    if (*s == '{')
-                        braces++;
-                    if (*s == '}')
-                        braces--;
-                    s++;
-                }
+            	t=getStringBraceParam(&s);
+            	free(t);
                 break;
             case 'p':
-                diagnostics(WARNING, " 'p{width}' not fully supported");
-                simple[i++] = 'l';
+                simple[iCol] = 'l';
+            	t=getStringBraceParam(&s);
+            	width = getStringDimension(t);
+            	free(t);
+                iCol++;
                 break;
             case '*':
-                diagnostics(WARNING, " '*{num}{cols}' not supported.\n");
+                diagnostics(WARNING, " '*{num}{cols}' not supported.");
                 break;
             case '@':
-                diagnostics(WARNING, " '@{text}' not supported.\n");
+                if (!warned_once) diagnostics(WARNING, " '@{text}' not supported.");
+                warned_once = TRUE;
                 break;
             default:
                 break;
         }
         s++;
     }
-    simple[i] = '\0';
-    diagnostics(4, "Exiting ConvertFormatString, output=<%s>", simple);
+    simple[iCol] = '\0';
+    diagnostics(5, "Exiting ConvertFormatString, output=<%s>", simple);
     return simple;
 }
 
-static int TabularColumnPosition(int n)
+static int TabularColumnPosition(TabularT tabular, int n)
 
 /******************************************************************************
  purpose:  return position of nth column 
  ******************************************************************************/
 {
-    int colWidth = getLength("textwidth") / colCount;
+    int colWidth = getLength("textwidth") / tabular.n;
 
     return colWidth * (n + 1);
 }
@@ -159,7 +158,7 @@ static void TabularCountVert(char *s, int vert[])
 {
     int braces, i;
 
-    diagnostics(4, "Entering TabularCountVert, input=<%s>", s);
+    diagnostics(5, "Entering TabularCountVert, input=<%s>", s);
 
     i = 0;
     while (*s) {
@@ -190,7 +189,7 @@ static void TabularCountVert(char *s, int vert[])
         }
         s++;
     }
-    diagnostics(4, "Exiting TabularCountVert");
+    diagnostics(5, "Exiting TabularCountVert");
 }
 
 static TabularT TabularPreamble(char *text, char *width, char *pos, char *cols)
@@ -199,40 +198,42 @@ static TabularT TabularPreamble(char *text, char *width, char *pos, char *cols)
  purpose:  calculate column widths 
  ******************************************************************************/
 {
-    TabularT tab;
-    int i;
+    TabularT tabular;
+    int i, colCount;
+    char *colFmt;
 
     colFmt = ConvertFormatString(cols);
     colCount = (int) strlen(colFmt);
-    actCol = 0;
 
-    tab.n = colCount;
-    tab.align = (char *) calloc(sizeof(char) * (colCount + 1), sizeof(char));
-    tab.vert = (int *) calloc(sizeof(int) * (colCount + 1), sizeof(int));
-    tab.width = (int *) calloc(sizeof(int) * (colCount + 1), sizeof(int));
-    tab.cline = (int *) calloc(sizeof(int) * (colCount + 1), sizeof(int));
+    tabular.i = 0;
+    tabular.n = colCount;
+    tabular.align = (char *) calloc(sizeof(char) * (colCount + 1), sizeof(char));
+    tabular.vert  = (int *)  calloc(sizeof(int)  * (colCount + 1), sizeof(int));
+    tabular.width = (int *)  calloc(sizeof(int)  * (colCount + 1), sizeof(int));
+    tabular.cline = (int *)  calloc(sizeof(int)  * (colCount + 1), sizeof(int));
 
     for (i = 1; i <= colCount; i++)
-        tab.align[i] = colFmt[i - 1];
+        tabular.align[i] = colFmt[i - 1];
 
-    TabularCountVert(cols, tab.vert);
+    TabularCountVert(cols, tabular.vert);
 
-    if (GetTexMode() != MODE_HORIZONTAL) {
+    if (getTexMode() != MODE_HORIZONTAL) {
         CmdIndent(INDENT_NONE);
-        CmdStartParagraph(FIRST_PAR);
+        startParagraph("tabular", FIRST_PARAGRAPH);
     }
 
     fprintRTF("\\par\n");
-    return tab;
+    
+    return tabular;
 }
 
 static void TabularGetRow(char *table, char **row, char **next_row, int *height)
 
 /******************************************************************************
  purpose:  scan and duplicate the next row from a tabular environment and any height changes
- 			e.g.   the & cell & is & here \\[2pt]
- 			       but & then & it & died \\
- 			will return "the & cell & is & here" and height should be 40 (twips)
+            e.g.   the & cell & is & here \\[2pt]
+                   but & then & it & died \\
+            will return "the & cell & is & here" and height should be 40 (twips)
  ******************************************************************************/
 {
     char *s, *dimension, *dim_start;
@@ -264,7 +265,7 @@ static void TabularGetRow(char *table, char **row, char **next_row, int *height)
     strncpy(*row, table, row_chars);
     (*row)[row_chars] = '\0';
 
-    diagnostics(4, "row =<%s>", *row);
+    show_string(5, *row, "row");
     if (!slashslash)
         return;
 
@@ -299,12 +300,9 @@ static void TabularGetRow(char *table, char **row, char **next_row, int *height)
     dimension[dim_chars] = '\n';    /* make sure entire string is not parsed */
     dimension[dim_chars + 1] = '\0';
 
-    if (PushSource(NULL, dimension) == 0) {
-        *height = getDimension();
-        PopSource();
-    }
+    *height = getStringDimension(dimension);
 
-    diagnostics(3, "height =<%s>=%d twpi", dimension, height);
+    diagnostics(5, "height =<%s>=%d twpi", dimension, height);
     free(dimension);
 
     /* skip blanks */
@@ -367,6 +365,7 @@ static void TabularMultiParameters(char *cell, int *col_span, char *align, int *
  ******************************************************************************/
 {
     char *p, *format, *mformat, *num;
+    diagnostics(5, "TabularMultiParameters cell=\"%s\" ", cell);
 
     *col_span = 0;
     *align = '\0';
@@ -377,10 +376,9 @@ static void TabularMultiParameters(char *cell, int *col_span, char *align, int *
     if (p == NULL)
         return;
 
-    PushSource(NULL, p + strlen("\\multicolumn"));
-    num = getBraceParam();
-    format = getBraceParam();
-    PopSource();
+	p      += strlen("\\multicolumn");
+	num     = getStringBraceParam(&p);
+	format  = getStringBraceParam(&p);
     mformat = ConvertFormatString(format);
 
     /* count '|' to the left of the column */
@@ -522,6 +520,7 @@ static void TabularBeginRow(TabularT tabular, char *this_row, char *next_row, in
 
     cline = TabularCline(next_row, tabular.n);
 
+
     while (cell_start) {        /* for each cell */
 
         top = 0;
@@ -535,6 +534,7 @@ static void TabularBeginRow(TabularT tabular, char *this_row, char *next_row, in
 
         cell = TabularNextCell(cell_start, &cell_end);
         TabularMultiParameters(cell, &n, &align, &lvert, &rvert);
+        
         if (n > 1)
             fprintRTF("\\clmgf");
 
@@ -554,7 +554,7 @@ static void TabularBeginRow(TabularT tabular, char *this_row, char *next_row, in
             fprintRTF("\\clbrdrr\\brdrs");
         if (right == 2 || (n == 1 && rvert == 2))
             fprintRTF("\\clbrdrr\\brdrdb");
-        fprintRTF("\\cellx%d", TabularColumnPosition(column));
+        fprintRTF("\\cellx%d", TabularColumnPosition(tabular, column));
         column++;
 
         for (i = 2; i <= n; i++) {
@@ -571,7 +571,7 @@ static void TabularBeginRow(TabularT tabular, char *this_row, char *next_row, in
                 fprintRTF("\\clbrdrr\\brdrs");
             if (i == n && rvert == 2)
                 fprintRTF("\\clbrdrr\\brdrdb");
-            fprintRTF("\\cellx%d", TabularColumnPosition(column));
+            fprintRTF("\\cellx%d", TabularColumnPosition(tabular, column));
             column++;
         }
 
@@ -582,7 +582,7 @@ static void TabularBeginRow(TabularT tabular, char *this_row, char *next_row, in
     free(cline);
 }
 
-static void TabularEndRow(void)
+void TabularEndRow(void)
 
 /******************************************************************************
  purpose:  emit RTF to finish one row of a table
@@ -591,13 +591,13 @@ static void TabularEndRow(void)
     fprintRTF("\\row\n");
 }
 
-char TabularColumnAlignment(int column)
+char TabularColumnAlignment(TabularT tabular, int column)
 
 /******************************************************************************
  purpose:  alignment for a column
  ******************************************************************************/
 {
-    return colFmt[column];
+    return tabular.align[column+1];
 }
 
 static void TabularWriteRow(TabularT tabular, char *this_row, char *next_row, int height, int first_row)
@@ -606,18 +606,25 @@ static void TabularWriteRow(TabularT tabular, char *this_row, char *next_row, in
     char align;
     int n, lvert, rvert;
 
-    actCol = 0;
+    tabular.i = 0;
     if (this_row == NULL || strlen(this_row) == 0)
         return;
 
-    diagnostics(4, "TabularWriteRow height=%d twpi, row <%s>", height, this_row);
+    diagnostics(5, "TabularWriteRow height=%d twpi, row <%s>", height, this_row);
 
     /* avoid writing anything for empty last row */
     if (next_row == NULL) {
-        if (colCount == 1 && strlen(this_row) == 0)
-            return;
-        if (colCount > 1 && !strchr(this_row, '&'))
+    	/* do nothing if the row is empty */
+        if (tabular.n == 1 && strlen(this_row) == 0)
             return;
+            
+        /* do nothing if there is more than one column, but no '&' */
+        if (tabular.n > 1 && !strchr(this_row, '&')) {
+        
+        	/* except if the last line is multicolumn! */
+            TabularMultiParameters(this_row, &n, &align, &lvert, &rvert);
+            if (n==0) return;
+		}
     }
 
     TabularBeginRow(tabular, this_row, next_row, first_row);
@@ -629,29 +636,29 @@ static void TabularWriteRow(TabularT tabular, char *this_row, char *next_row, in
         /* establish cell alignment */
         TabularMultiParameters(cell, &n, &align, &lvert, &rvert);
         if (n == 0) {
-            align = TabularColumnAlignment(actCol);
+            align = TabularColumnAlignment(tabular, tabular.i);
             n = 1;
         }
 
-        BeginCell(align);
+        BeginCellRtf(align);
         if (cell != NULL) {
             diagnostics(5, "TabularWriteRow align=%c n=%d cell=<%s>", align, n, cell);
             fprintRTF("{");
             ConvertString(cell);
             fprintRTF("}");
         }
-        EndCell();
+        EndCellRtf();
 
-        actCol += n;
+        tabular.i += n;
         cell_start = cell_end;
         if (cell != NULL)
             free(cell);
     }
 
-    if (actCol < colCount) {
-        align = TabularColumnAlignment(actCol);
-        BeginCell(align);
-        EndCell();
+    if (tabular.i < tabular.n) {
+        align = TabularColumnAlignment(tabular, tabular.i);
+        BeginCellRtf(align);
+        EndCellRtf();
     }
 
     TabularEndRow();
@@ -749,11 +756,11 @@ static void TabularMeasureRow(TabularT tabular, char *this_row, char *next_row,
     char align;
     int n, lvert, rvert;
 
-    actCol = 0;
+    tabular.i = 0;
     if (this_row == NULL || strlen(this_row) == 0)
         return;
 
-    diagnostics(4, "TabularMeasureRow height=%d twpi, row <%s>", height, this_row);
+    diagnostics(5, "TabularMeasureRow height=%d twpi, row <%s>", height, this_row);
 
     cell_start = this_row;
     while (cell_start) {
@@ -769,10 +776,10 @@ static void TabularMeasureRow(TabularT tabular, char *this_row, char *next_row,
             int len;
 
             len = TabularMeasureCell(cell);
-            diagnostics(6, "col=%d n=%d len=%d cell=<%s>", actCol, n, len, cell);
+            diagnostics(6, "col=%d n=%d len=%d cell=<%s>", tabular.i, n, len, cell);
         }
 
-        actCol += n;
+        tabular.i += n;
         cell_start = cell_end;
         if (cell != NULL)
             free(cell);
@@ -782,9 +789,9 @@ static void TabularMeasureRow(TabularT tabular, char *this_row, char *next_row,
 void CmdTabular(int code)
 
 /******************************************************************************
- purpose: 	\begin{tabular}[pos]{cols}          ... \end{tabular}
- 			\begin{tabular*}{width}[pos]{cols}  ... \end{tabular*}
- 			\begin{array}[pos]{cols}            ... \end{array}
+ purpose:   \begin{tabular}[pos]{cols}          ... \end{tabular}
+            \begin{tabular*}{width}[pos]{cols}  ... \end{tabular*}
+            \begin{array}[pos]{cols}            ... \end{array}
  ******************************************************************************/
 {
     int true_code, this_height, next_height, first_row, begins, ends;
@@ -793,14 +800,12 @@ void CmdTabular(int code)
     char *cols = NULL;
     char *pos = NULL;
     char *width = NULL;
+    
     TabularT tabular;
 
     if (!(code & ON)) {
         diagnostics(4, "Exiting CmdTabular");
         g_processing_tabular = FALSE;
-        if (colFmt)
-            free(colFmt);
-        colFmt = NULL;
         return;
     }
 
@@ -834,90 +839,104 @@ void CmdTabular(int code)
     cols = getBraceParam();
     table = getTexUntil(end, FALSE);
 
-/* need to make sure that we don't have nested environments */
-    begins = strstr_count(table, begin);
-    ends = strstr_count(table, end);
+	if (g_tabular_display_bitmap) {
+		char pre[151];
+		
+		snprintf(pre,150, "%s{%s}",begin,cols);
+		PrepareDisplayedBitmap("tabular");
+		WriteLatexAsBitmap(pre, table, end);
+		FinishDisplayedBitmap();
+	}
+	
+	if (g_tabular_display_rtf)
+	{
+	/* need to make sure that we don't have nested environments */
+		begins = strstr_count(table, begin);
+		ends = strstr_count(table, end);
+	
+		while (begins > ends) {
+			char *table2, *table3, *table4;
+	
+			if (begins > ends) {
+				table2 = getTexUntil(end, FALSE);
+				table3 = strdup_together(table, end);
+				table4 = strdup_together(table3, table2);
+				free(table);
+				free(table2);
+				free(table3);
+				table = table4;
+			}
+			begins = strstr_count(table, begin);
+			ends = strstr_count(table, end);
+		}
+	
+		if (begins > 0) {
+			char *p;
+			int num = TexFontNumber("Typewriter");
+	
+			diagnostics(WARNING, "Nested tabular/tabbing environments not allowed");
+			diagnostics(5, "table_table_table_table_table\n%stable_table_table_table_table", table);
+			fprintRTF("\\pard\\ql\\b0\\i0\\scaps0\\f%d ", num);
+			p = begin;
+			while (*p)
+				fprintRTF("%c",*p++);
+			p = table;
+			while (*p)
+				fprintRTF("%c",*p++);
+			p = end;
+			while (*p)
+				fprintRTF("%c",*p++);
+	
+		} else {
+	
+			diagnostics(4, "Entering CmdTabular() options [%s], format {%s}", (pos) ? pos : "", cols);
+			tabular = TabularPreamble(table, width, pos, cols);
+			diagnostics(5, "table_table_table_table_table\n%stable_table_table_table_table", table);
+	
+			row_start = table;
+			TabularGetRow(row_start, &this_row, &next_row_start, &this_height);
+	
+			/* scan entire table to get estimates for column widths */
+			first_row = TRUE;
+			while (this_row) {
+				row_start = next_row_start;
+				TabularGetRow(row_start, &next_row, &next_row_start, &next_height);
+				TabularMeasureRow(tabular, this_row, next_row, this_height, first_row);
+				free(this_row);
+				this_row = next_row;
+				this_height = next_height;
+				first_row = FALSE;
+			}
+	
+			row_start = table;
+			TabularGetRow(row_start, &this_row, &next_row_start, &this_height);
+			first_row = TRUE;
+			while (this_row) {
+				row_start = next_row_start;
+				TabularGetRow(row_start, &next_row, &next_row_start, &next_height);
+				TabularWriteRow(tabular, this_row, next_row, this_height, first_row);
+				free(this_row);
+				this_row = next_row;
+				this_height = next_height;
+				first_row = FALSE;
+			}
+	
+			free(tabular.cline);
+			free(tabular.align);
+			free(tabular.vert);
+			free(tabular.width);
+			if (cols)
+				free(cols);
+			if (pos)
+				free(pos);
+			if (width)
+				free(width);
+		}
+	}
 
-    while (begins > ends) {
-        char *table2, *table3, *table4;
-
-        if (begins > ends) {
-            table2 = getTexUntil(end, FALSE);
-            table3 = strdup_together(table, end);
-            table4 = strdup_together(table3, table2);
-            free(table);
-            free(table2);
-            free(table3);
-            table = table4;
-        }
-        begins = strstr_count(table, begin);
-        ends = strstr_count(table, end);
-    }
-
-    if (begins > 0) {
-        char *p;
-        int num = TexFontNumber("Typewriter");
-
-        diagnostics(1, "Nested tabular/tabbing environments not allowed");
-        diagnostics(2, "table_table_table_table_table\n%stable_table_table_table_table", table);
-        fprintRTF("\\pard\\ql\\b0\\i0\\scaps0\\f%d ", num);
-        p = begin;
-        while (*p)
-            putRtfChar(*p++);
-        p = table;
-        while (*p)
-            putRtfChar(*p++);
-        p = end;
-        while (*p)
-            putRtfChar(*p++);
-
-    } else {
-
-        diagnostics(3, "Entering CmdTabular() options [%s], format {%s}", (pos) ? pos : "", cols);
-        tabular = TabularPreamble(table, width, pos, cols);
-        diagnostics(2, "table_table_table_table_table\n%stable_table_table_table_table", table);
-
-        row_start = table;
-        TabularGetRow(row_start, &this_row, &next_row_start, &this_height);
-
-        /* scan entire table to get estimates for column widths */
-        first_row = TRUE;
-        while (this_row) {
-            row_start = next_row_start;
-            TabularGetRow(row_start, &next_row, &next_row_start, &next_height);
-            TabularMeasureRow(tabular, this_row, next_row, this_height, first_row);
-            free(this_row);
-            this_row = next_row;
-            this_height = next_height;
-            first_row = FALSE;
-        }
-
-        row_start = table;
-        TabularGetRow(row_start, &this_row, &next_row_start, &this_height);
-        first_row = TRUE;
-        while (this_row) {
-            row_start = next_row_start;
-            TabularGetRow(row_start, &next_row, &next_row_start, &next_height);
-            TabularWriteRow(tabular, this_row, next_row, this_height, first_row);
-            free(this_row);
-            this_row = next_row;
-            this_height = next_height;
-            first_row = FALSE;
-        }
-
-        free(tabular.cline);
-        free(tabular.align);
-        free(tabular.vert);
-        free(tabular.width);
-        if (cols)
-            free(cols);
-        if (pos)
-            free(pos);
-        if (width)
-            free(width);
-    }
-
-    ConvertString(end);
+	ConvertString(end);
+	
+	
     free(table);
     free(end);
     free(begin);
@@ -934,10 +953,28 @@ static int TabbingColumnPosition(int n, int total)
     return colWidth * (n + 1);
 }
 
+/******************************************************************************
+ purpose:  simple minded way to skip \verb#contents$# in cell
+ ******************************************************************************/
+static char *skip_verb(char *s)
+{
+    char endchar;
+    diagnostics(6,"before verb <<%s>>",s);
+    if (s && strncmp(s,"verb",4)==0){
+        s+=4;
+        endchar = *s;
+        s++;
+        while (*s && *s!= endchar) s++;
+        if (*s) s++;
+    }
+    diagnostics(6,"after  verb <<%s>>",s);
+    return s;
+}
+
 static void TabbingNextCellEnd(char *t, char **cell_end, char **next_cell)
 
 /******************************************************************************
- purpose:  find the next ampersand while avoiding \&
+ purpose:  find the end of this tabbing cell
  ******************************************************************************/
 {
     char *s;
@@ -954,12 +991,19 @@ static void TabbingNextCellEnd(char *t, char **cell_end, char **next_cell)
 
         if (*s == '\\') {
             s++;
+            s = skip_verb(s);
             if (*s == '=' || *s == '>' || *s == '<' || *s == '\'' || *s == '`') {
                 *cell_end = s - 1;
                 *next_cell = s + 1;
                 return;
             }
         }
+        
+        if (*s == '\0') {
+            *cell_end = s;
+            *next_cell = s;
+            return;
+        }
         s++;
     }
 }
@@ -992,6 +1036,7 @@ static char *TabbingNextCell(char *cell_start, char **cell_end)
 
     dup2 = strdup_noendblanks(dup);
     free(dup);
+    diagnostics(5,"next cell = [[%s]]", dup);
     return dup2;
 }
 
@@ -1013,8 +1058,8 @@ static void TabbingBeginRow(int n, int n_total, char *align)
 
     fprintRTF("\n");
     for (i = 0; i < g_tabbing_left_position; i++) {
-        BeginCell(align[i]);
-        EndCell();
+        BeginCellRtf(align[i]);
+        EndCellRtf();
     }
 
 }
@@ -1024,10 +1069,11 @@ static void TabbingWriteRow(char *this_row, int n, int n_total, char *align)
     char *start, *end, *cell;
     int i;
 
+    diagnostics(5, "TabbingWriteRow n=%d <%s> [%s]", n, align, this_row);
+
     if (this_row == NULL || n == 0)
         return;
 
-    diagnostics(4, "TabbingWriteRow n=%d <%s> [%s]", n, align, this_row);
     if (strstr(this_row, "\\kill"))
         return;
 
@@ -1037,14 +1083,14 @@ static void TabbingWriteRow(char *this_row, int n, int n_total, char *align)
     end = this_row + strlen(this_row);
 
     for (i = g_tabbing_left_position; i < n; i++) {
-        BeginCell(align[i]);
+        BeginCellRtf(align[i]);
         cell = TabbingNextCell(start, &end);
         if (cell) {
-            diagnostics(4, "cell=<%s>", cell);
+            diagnostics(5, "cell=<%s>", cell);
             ConvertString(cell);
             free(cell);
         }
-        EndCell();
+        EndCellRtf();
         start = end;
     }
     fprintRTF("\\row\n");
@@ -1058,7 +1104,7 @@ static void TabbingGetRow(char *table, char **row, char **next_row)
 {
     char *s, *arow;
     size_t row_chars = 0;
-    bool slash = FALSE;
+    bool slash;
 
     s = table;
     *row = NULL;
@@ -1067,25 +1113,40 @@ static void TabbingGetRow(char *table, char **row, char **next_row)
     if (!s)
         return;
 
-    /* the end of a row is caused by one of three things 1) the buffer ends 2) the line ends \\ 3) \kill is encountered 
+    /* the end of a row is caused by one of three things 
+            1) the buffer ends 
+            2) the line ends \\ 
+            3) \kill is encountered 
      */
+     
+    slash = FALSE;
     while (!(*s == '\0') &&
-      !(*s == '\\' && slash) && !((row_chars > 6) && strncmp(s - 5, "\\kill", 5) == 0 && !isalpha((int) *s))
-      ) {
+           !(*s == '\\' && slash) && 
+           !((row_chars > 6) && strncmp(s - 5, "\\kill", 5) == 0 && !isalpha((int) *s))
+          ) {
         row_chars++;
         slash = (*s == '\\') ? 1 : 0;
         s++;
     }
 
+    if (*s == '\\' && slash) {  /* line ends with \\ */
+        row_chars--;
+        
+        if (*(s+1)=='[') {  /* skip optional height parameter */
+        	s++;
+        	while (*s != '\0' && *s != ']') s++;
+        }
+    }
+
     if (*s != '\0')
         *next_row = s + 1;
-    if (*s == '\\' && slash)
-        row_chars--;
 
     arow = (char *) malloc((row_chars + 1) * sizeof(char));
     strncpy(arow, table, row_chars);
     arow[row_chars] = '\0';
 
+    diagnostics(5, "TabbingGetRow obtained=<%s> remaining[%s]", arow, *next_row);
+
     *row = strdup_noendblanks(arow);
     free(arow);
 }
@@ -1093,7 +1154,7 @@ static void TabbingGetRow(char *table, char **row, char **next_row)
 static void TabbingGetColumnAlignments(char *row, char *align, int *n, int *next_left)
 
 /******************************************************************************
- purpose: 	scan one row of tabbing environment to obtain column alignments
+ purpose:   scan one row of tabbing environment to obtain column alignments
  ******************************************************************************/
 {
     int i;
@@ -1112,6 +1173,8 @@ static void TabbingGetColumnAlignments(char *row, char *align, int *n, int *next
         }
 
         row++;
+        row = skip_verb(row);
+        
         switch (*row) {
 
             case '=':
@@ -1157,7 +1220,7 @@ static void TabbingGetColumnAlignments(char *row, char *align, int *n, int *next
 void CmdTabbing(int code)
 
 /******************************************************************************
- purpose: 	\begin{tabbing} ... \end{tabbing}
+ purpose:   \begin{tabbing} ... \end{tabbing}
  ******************************************************************************/
 {
     int n, n_total, next_left;
@@ -1168,9 +1231,6 @@ void CmdTabbing(int code)
     if (!(code & ON)) {
         diagnostics(4, "Exiting CmdTabbing");
         g_processing_tabbing = FALSE;
-        if (colFmt)
-            free(colFmt);
-        colFmt = NULL;
         return;
     }
 
@@ -1181,40 +1241,48 @@ void CmdTabbing(int code)
 
     end = strdup("\\end{tabbing}");
     table = getTexUntil(end, FALSE);
-    diagnostics(3, "Entering CmdTabbing()");
-
-    row_start = table;
-    TabbingGetRow(row_start, &this_row, &next_row_start);
-
-    diagnostics(2, "tabbing_tabbing_tabbing\n%s\ntabbing_tabbing_tabbing", table);
-
-    if (GetTexMode() != MODE_HORIZONTAL) {
-        CmdIndent(INDENT_NONE);
-        CmdStartParagraph(FIRST_PAR);
-    }
-
-    fprintRTF("\\par\n");
-
-    n_total = 0;
-    while (this_row && strlen(this_row) > 0) {
-        diagnostics(4, "row=<%s>", this_row);
-
-        TabbingGetColumnAlignments(this_row, align, &n, &next_left);
-        if (n > n_total)
-            n_total = n;
-
-        diagnostics(4, "this row n=%d <%s> left_tab=%d", n, align, g_tabbing_left_position);
-
-        TabbingWriteRow(this_row, n, n_total, align);
-
-        g_tabbing_left_position = next_left;
-        row_start = next_row_start;
-
-        free(this_row);
-        TabbingGetRow(row_start, &this_row, &next_row_start);
-    }
+    diagnostics(4, "Entering CmdTabbing()");
 
+	if (g_tabular_display_bitmap) {		
+		PrepareDisplayedBitmap("tabbing");
+		WriteLatexAsBitmap("\\begin{tabbing}", table, end);
+		FinishDisplayedBitmap();
+	}
+	
+	if (g_tabular_display_rtf) {
+		row_start = table;
+		TabbingGetRow(row_start, &this_row, &next_row_start);
+	
+		diagnostics(5, "tabbing_tabbing_tabbing\n%s\ntabbing_tabbing_tabbing", table);
+	
+		if (getTexMode() != MODE_HORIZONTAL) {
+			CmdIndent(INDENT_NONE);
+			startParagraph("tabbing", FIRST_PARAGRAPH);
+		}
+	
+		fprintRTF("\\par\n");
+	
+		n_total = 0;
+		while (this_row && strlen(this_row) >= 0) {
+			show_string(5, this_row, "row");
+	
+			TabbingGetColumnAlignments(this_row, align, &n, &next_left);
+			if (n > n_total)
+				n_total = n;
+	
+			diagnostics(4, "this row n=%d <%s> left_tab=%d", n, align, g_tabbing_left_position);
+	
+			TabbingWriteRow(this_row, n, n_total, align);
+	
+			g_tabbing_left_position = next_left;
+			row_start = next_row_start;
+	
+			free(this_row);
+			TabbingGetRow(row_start, &this_row, &next_row_start);
+		}
+	}
     ConvertString(end);
+    
 
     free(table);
     free(end);
@@ -1223,47 +1291,70 @@ void CmdTabbing(int code)
 void CmdTable(int code)
 
 /******************************************************************************
- purpose: converts the LaTex-Table to a similar Rtf-style
-	  this converting is only partially
-	  so the user has to convert some part of the Table-environment by hand
-parameter: type of array-environment
+ purpose: handles the \begin{table} ... \end{table} codes.  The primary use is
+          to properly handle the spacing before and after the table.  It is 
+          slightly more complicated because the endfloat package suppresses 
+          inserting a table in the text, but does it later when the file.ttt 
+          is read.  g_endfloat_tables is set to true during this phase --- 
+          otherwise it is always false (most notably when the endfloat package
+          is not being used.)
  ******************************************************************************/
 {
     char *location, *table_contents;
-    char endtable[] = "\\end{table}";
-	static char     oldalignment;
-
+    static char     oldalignment;
+    int true_code = code & ~ON;
+		
     if (code & ON) {
-		CmdEndParagraph(0);
-		oldalignment = alignment;
-		alignment = JUSTIFIED;
-
-		CmdVspace(VSPACE_BIG_SKIP);
-		CmdStartParagraph(0);
-
         location = getBracketParam();
-        if (location)
-            free(location);
+        if (location) free(location);
 
-        if (GetTexMode() != MODE_VERTICAL)
-            CmdEndParagraph(0);
+        CmdEndParagraph(0);
+        oldalignment = getAlignment();
+        setAlignment(JUSTIFIED);
+
+        CmdVspace(VSPACE_BIG_SKIP);
         CmdIndent(INDENT_NONE);
 
         g_processing_table = TRUE;
-        table_contents = getTexUntil(endtable, TRUE);
+        
+        if (true_code == TABLE)
+        	table_contents = getTexUntil("\\end{table}", TRUE);
+        else
+        	table_contents = getTexUntil("\\end{table*}", TRUE);
+        	
         g_table_label = ExtractLabelTag(table_contents);
-        ConvertString(table_contents);
-        ConvertString(endtable);
+        if (g_endfloat_tables) {
+            if (g_endfloat_markers) {
+                setAlignment(CENTERED);
+                startParagraph("endfloat", GENERIC_PARAGRAPH);
+                incrementCounter("endfloattable");  /* two separate counters */
+                fprintRTF("[");                     /* one for tables and one for */
+                ConvertBabelName("TABLENAME");      /* endfloat tables */
+                fprintRTF(" ");
+                if (g_document_type != FORMAT_ARTICLE &&
+                    g_document_type != FORMAT_APA)
+                    fprintRTF("%d.", getCounter("chapter"));
+                fprintRTF("%d about here]", getCounter("endfloattable"));
+            }
+        } else {
+            startParagraph("table", GENERIC_PARAGRAPH);
+            ConvertString(table_contents);
+        }
         free(table_contents);
+        
+        if (true_code == TABLE)
+        	ConvertString("\\end{table}");
+        else
+        	ConvertString("\\end{table*}");
     } else {
         g_processing_table = FALSE;
-        if (GetTexMode() != MODE_VERTICAL)
+        if (getTexMode() != MODE_VERTICAL)
             CmdEndParagraph(0);
         if (g_table_label)
             free(g_table_label);
-		alignment = oldalignment;
-		CmdEndParagraph(0);
-		CmdVspace(VSPACE_BIG_SKIP);
+        setAlignment(oldalignment);
+        CmdEndParagraph(0);
+        CmdVspace(VSPACE_BIG_SKIP);
     }
 }
 
